<?php
/**
 * @brief		Chatbox+ Application Class
 * @author		<a href='https://ipsviet.com'>onlyME</a>
 * @copyright	(c) 2018 onlyME
 * @package		Invision Community
 * @subpackage	Chatbox+
 * @since		18 Sep 2018
 * @version		
 */
 
namespace IPS\chatbox;

/**
 * Chatbox+ Application Class
 */
class _Application extends \IPS\Application
{
	//===========================================================================
	// [Node] Get Icon
	//===========================================================================	
	protected function get__icon()
	{
		return 'comments';
	}	
	
	/**
	 * Default front navigation
	 *
	 * @code
	 	
	 	// Each item...
	 	array(
			'key'		=> 'Example',		// The extension key
			'app'		=> 'core',			// [Optional] The extension application. If ommitted, uses this application	
			'config'	=> array(...),		// [Optional] The configuration for the menu item
			'title'		=> 'SomeLangKey',	// [Optional] If provided, the value of this language key will be copied to menu_item_X
			'children'	=> array(...),		// [Optional] Array of child menu items for this item. Each has the same format.
		)
	 	
	 	return array(
		 	'rootTabs' 		=> array(), // These go in the top row
		 	'browseTabs'	=> array(),	// These go under the Browse tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'browseTabsEnd'	=> array(),	// These go under the Browse tab after all other items on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'activityTabs'	=> array(),	// These go under the Activity tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Activity tab may not exist)
		)
	 * @endcode
	 * @return array
	 */
	public function defaultFrontNavigation()
	{
		return array(
			'rootTabs'		=> array(),
			'browseTabs'	=> array( array( 'key' => 'Chatbox' ) ),
			'browseTabsEnd'	=> array(),
			'activityTabs'	=> array()
		);
	}	
	
	/*-------------------------------------------------------------------------*/
	// Styles & Skins
	/*-------------------------------------------------------------------------*/	
	public function styles()
	{
		return array(
			'standard'		=> 'chatbox_standard',
			'condensed'		=> 'chatbox_condensed',
			'bubbles'		=> 'chatbox_bubbles',
		);
	}	

	public function skins()
	{
		return array( 'white', 'black', 'aqua', 'maroon', 'navy', 'olive' );
	}		
	
	/*-------------------------------------------------------------------------*/
	// Make clean Text
	/*-------------------------------------------------------------------------*/	
	protected $looseProfanity = array();
	
	protected $exactProfanity = array();

	protected $caseSensitiveAcronyms = array();

	protected $caseInsensitiveAcronyms = array();
	
	public function makeCleanText($txt)
	{
		# Remove tags
		$txt = htmlspecialchars( $txt, ENT_DISALLOWED, 'UTF-8', FALSE );

		# Img proxy
		if ( \IPS\Settings::i()->remote_image_proxy )
		{	
			preg_match("~https?://\S+\.(?:jpe?g|gif|png|webp)(?:\?\S*)?(?=\s|$|\pP)~i", $txt, $img);
			$imageSrc = new \IPS\Http\Url( $img[0] );
			if ( !$imageSrc->isInternal && !$imageSrc->isLocalhost() && $imageSrc->data[ \IPS\Http\Url::COMPONENT_SCHEME ] != 'https' ) 
			{
				$newUrl = \IPS\Http\Url::createFromString( \IPS\Settings::i()->base_url . "applications/core/interface/imageproxy/imageproxy.php" );
			
				$newUrl = $newUrl->setQueryString( array(
					'key'	=> hash_hmac( "sha256", (string) $imageSrc, \IPS\Settings::i()->site_secret_key ),
					'img' 	=> str_replace( ',', '%2C', (string) $imageSrc ) /* srcset URLs can have a param like resize=150,150 and the comma breaks the URLs */
				) );				
				$newIMG = (string) $newUrl;
				$txt = str_replace($img[0], $newIMG, $txt);
			}
		}
		
		# Bad word
		foreach( \IPS\core\Profanity::getProfanity() AS $profanity )
		{
			if ( $profanity->action == 'swap' )
			{
				if ( $profanity->m_exact )
				{
					$this->exactProfanity[ $profanity->type ] = $profanity->swop;
				}
				else
				{
					$this->looseProfanity[ $profanity->type ] = $profanity->swop;
				}
			}
		}
	
		foreach ( $this->exactProfanity as $bad => $good )
		{
			$txt = preg_replace( '/(^|\b|\s)' . preg_quote( $bad, '/' ) . '(\b|\s|!|\?|\.|,|$)/iu', "\\1" . $good . "\\2", $txt );
		}
		$txt = str_ireplace( array_keys( $this->looseProfanity ), array_values( $this->looseProfanity ), $txt );

			
		# Done
		return $txt;
	}

	/*-------------------------------------------------------------------------*/
	// Replace emoji
	/*-------------------------------------------------------------------------*/		
	public function replaceEmoji( $output, $withImage=true )
	{
		return preg_replace_callback( '/' . \IPS\Text\Parser::EMOJI_REGEX . '/u', function( $matches ) use ( $withImage ) {
			
			if ( !$withImage ) 
			{
				return '';
			}
			$hex = bin2hex( mb_convert_encoding( $matches[0], 'UTF-32', 'UTF-8' ) );
			$hexLength = \strlen( $hex ) / 8;
			$chunks = array();
			for ( $i = 0; $i < $hexLength; ++$i )
			{
				$tmp = \substr( $hex, $i * 8, 8 );
						
				$copy = false;
				$len = \strlen( $tmp );
				$res = '';
				for ( $j = 0; $j < $len; ++$j )
				{
					$ch = $tmp[ $j ];
					if ( !$copy )
					{
						if ( $ch != '0' )
						{
							$copy = true;
						}
						else if ( ( $i + 1 ) == $len )
						{
							$res = '0';
						}
					}
					if ( $copy )
					{
						$res .= $ch;
					}
				}
				
				$chunks[ $i ] = $res;
			}

			$image = implode( '-', $chunks );
			
			if ( \strstr( $image, '200d' ) === FALSE or $image === '1f441-fe0f-200d-1f5e8-fe0f' )
			{
				$image = str_replace( '-fe0f', '', $image );
			}
			if ( \in_array( $image, array( '0031-20e3', '0030-20e3', '0032-20e3', '0034-20e3', '0035-20e3', '0036-20e3', '0037-20e3', '0038-20e3', '0033-20e3', '0039-20e3', '0023-20e3', '002a-20e3', '00a9', '00ae' ) ) )
			{
				$image = str_replace( '00', '', $image );
			}

			return 'https://twemoji.maxcdn.com/2/72x72/'  . $image . '.png';
			
		}, $output );
			
		return $output;
	}	

	/*-------------------------------------------------------------------------*/
	// IPS default Emoticons
	/*-------------------------------------------------------------------------*/
	protected $_ipsEmoticons = NULL;
	
	public function ipsEmoticons()
	{
		if ( $this->_ipsEmoticons !== NULL )
		{
			return $this->_ipsEmoticons;
		}
		
		$emoticons = array();
		foreach ( \IPS\Db::i()->select( '*', 'core_emoticons', NULL, 'emo_set,emo_position' ) as $row )
		{
			$emoticons[ $row['typed'] ] = (string) \IPS\File::get( 'core_Emoticons', $row['image'] )->url;
		}
		return $emoticons;
	}
	
	/*-------------------------------------------------------------------------*/
	// Chatbox bar
	/*-------------------------------------------------------------------------*/
	public function chatBar()
	{
		if ( \IPS\Dispatcher::i()->controllerLocation == 'front' && ! \IPS\Request::i()->isAjax() && \IPS\Request::i()->controller != 'room' )
		{
			$canShow = true;
			
			if ( isset( \IPS\Settings::i()->chatbox_bar_expages ) )
			{
				$inPage = (string) \IPS\Request::i()->url();

				foreach ( explode( "\n", str_replace( "\r", "", \IPS\Settings::i()->chatbox_bar_expages ) ) as $url )
				{
					if ( $url == $inPage || preg_match( "~^" . str_replace( '\*', '.*?', preg_quote( $url ) ) . "$~i", htmlspecialchars_decode( $inPage ) ) )
					{
						$canShow = false;
						break;
					}				
				}	
			}		
			
			if ( $canShow )
			{
				return \IPS\Theme::i()->getTemplate( 'global', 'chatbox', 'front' )->chatBar();
			}
		}
		
		return '';
	}
	
	
	/*-------------------------------------------------------------------------*/
	// create Title
	/*-------------------------------------------------------------------------*/		
	public function autoTitle( $msg )
	{
		$msg = strip_tags( $msg );
		$msg = preg_replace('/\r|\n/', ' ', $msg); // remove new lines
		$msg = preg_replace('/[\s]+/mu', ' ', $msg); // remove new lines
		$msg = trim( mb_substr( $msg, 0, 40 ) );

		return $msg;
	}	
}