//<?php

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

abstract class chatbox_hook_Item extends _HOOK_CLASS_
{
	
	/**
	 * @brief	Is new record?
	 */
	protected $_isNew = NULL;
	
	
	/**
	 * Save Changed Columns
	 *
	 * @return	void
	 */
	public function save()
	{
		try
		{
			$this->_isNew = ( $this->_new AND !isset( $this->_isNew ) ) ? TRUE : FALSE;
			
			parent::save();
			
			$idColumn = static::$databaseColumnId;
			
			if ( $this->_isNew && $this->$idColumn && !$this->hidden() && !$this->isFutureDate() )
			{
				\IPS\chatbox\Rooms\Room::addNotification( $this );
				$this->_isNew = FALSE;
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}

	/**
	 * Syncing to run when publishing something previously pending publishing
	 *
	 * @param	\IPS\Member|NULL|FALSE	$member	The member doing the action (NULL for currently logged in member, FALSE for no member)
	 * @return	void
	 */
	public function onPublish( $member )
	{
		try
		{
			parent::onPublish( $member );
			
			if ( ! $this->hidden() )
			{
				\IPS\chatbox\Rooms\Room::addNotification( $this );
			}		
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * Syncing to run when unhiding
	 *
	 * @param	bool					$approving	If true, is being approved for the first time
	 * @param	\IPS\Member|NULL|FALSE	$member	The member doing the action (NULL for currently logged in member, FALSE for no member)
	 * @return	void
	 */
	public function onUnhide( $approving, $member )
	{
		try
		{
			parent::onUnhide( $approving, $member );
			
			if ( $approving AND ! $this->isFutureDate() )
			{
				\IPS\chatbox\Rooms\Room::addNotification( $this );
			}	
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	
	
	
}
