<?php


namespace IPS\chatbox\modules\admin\chatbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * clubs
 */
class _clubs extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'clubs_manage' );
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'skins.css', 'chatbox', 'front' ) );
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$opts = json_decode( \IPS\Settings::i()->room_clubs_configs, TRUE );
		
		# Build Form
		$form = new \IPS\Helpers\Form;	
		
		$form->addTab( 'room_tab_general' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'room_clubs', \IPS\Settings::i()->room_clubs, FALSE ) );
		$form->add( new \IPS\Helpers\Form\Upload( 'room_clubs_notifile', \IPS\Settings::i()->room_clubs_notifile ? \IPS\File::get( 'chatbox_Nofifile',\IPS\Settings::i()->room_clubs_notifile ) : NULL, FALSE, array( 
			'allowedFileTypes'	=> array( 'mp3' ),
			'maxFileSize'		=> 1,
			'storageExtension' 	=> 'chatbox_Nofifile',
		), NULL, NULL, NULL, 'room_clubs_notifile' ) );

		$form->add( new \IPS\Helpers\Form\Select( 'chatbox_style', $opts ? $opts['style'] : 'standard', FALSE, array( 'options' => \IPS\Application::load('chatbox')->styles() ) ) );	
		foreach( \IPS\Application::load('chatbox')->skins() as $bg )
		{
			$skinClass = $bg != 'white' ? $bg . '_box' : 'ipsType_sectionTitle';
			$skinSelect[ $bg ] = "<div class='ipsBox {$skinClass}' style='height: 20px; width: 80px;'></div>";
		}		
		$form->add( new \IPS\Helpers\Form\Radio( 'chatbox_skin', $opts['skin'] ? $opts['skin'] : 'white', FALSE, array( 'options' => $skinSelect, 'parse' => 'raw' ) ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_allow_select_skins', $opts['allow_select_skins'], FALSE, array(), NULL, NULL, NULL, 'chatbox_allow_select_skins' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'chatbox_height', $opts['height'] ? \intval( $opts['height'] ) : 420, TRUE, array( 'min' => 200 ) ) );
		
		# Chat
		$form->addTab( 'room_tab_chat' );
		$form->add( new \IPS\Helpers\Form\Select( 'chatbox_sort', $opts ? $opts['sort'] : 'asc', FALSE, array( 'options' => array(
			'asc'	=> 'chatbox_asc_sort',
			'desc'	=> 'chatbox_desc_sort'
		) ) ) );		
		$form->add( new \IPS\Helpers\Form\Number( 'chatbox_limit', $opts['limit'] ? \intval( $opts['limit'] ) : 20, TRUE, array( 'min' => 5 ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'chatbox_interval', $opts['interval'] ? \intval( $opts['interval'] ) : 10000, TRUE, array( 'min' => 2000 ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'chatbox_flood', $opts['flood'] ? \intval( $opts['flood'] ) : 5, FALSE, array( 'unlimited' => 0, 'unlimitedLang' => 'none' ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'chatbox_length', $opts['length'] ? \intval( $opts['length'] ) : 300, FALSE, array( 'unlimited' => 0, 'unlimitedLang' => 'unlimited' ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_autoload', $opts['autoload'], FALSE ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_showUserList', $opts['showUserList'], FALSE ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_auto_guest', $opts['auto_guest'], FALSE ) );
		
		# Input
		$form->addTab('chatbox_tab_input_media');
		$form->addHeader('chatbox_tab_media');
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_musicbox', $opts['musicbox'], FALSE, array(), NULL, NULL, NULL, 'chatbox_musicbox' ) );

		$form->addHeader('chatbox_tab_input');
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_use_editor', $opts['use_editor'], FALSE, array(
			'togglesOff' => array( 'chatbox_allow_img', 'chatbox_allow_video', 'chatbox_giphy', 'chatbox_guggy' ),
			'togglesOn'	 => array( 'chatbox_enter_editor', 'chatbox_editor_upload' )
		) ) );	
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_editor_upload', $opts['editor_upload'], FALSE, array(), NULL, NULL, NULL, 'chatbox_editor_upload' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_enter_editor', $opts['enter_editor'], FALSE, array(), NULL, NULL, NULL, 'chatbox_enter_editor' ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_allow_img', $opts['allow_img'], FALSE, array(), NULL, NULL, NULL, 'chatbox_allow_img' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_allow_video', $opts['allow_video'], FALSE, array(), NULL, NULL, NULL, 'chatbox_allow_video' ) );
		$form->add( new \IPS\Helpers\Form\Select( 'chatbox_giphy', $opts['giphy'] ? $opts['giphy'] : 'no', FALSE, array( 'options' => array( 'no' => 'chatbox_giphy_no', 'button' => 'chatbox_giphy_button', 'cmd' => 'chatbox_giphy_cmd', 'both' => 'chatbox_giphy_both' ) ), NULL, NULL, NULL, 'chatbox_giphy' ) );	
		$form->add( new \IPS\Helpers\Form\Select( 'chatbox_guggy', $opts['guggy'] ? $opts['guggy'] : 'no', FALSE, array( 'options' => array( 'no' => 'chatbox_guggy_no', 'button' => 'chatbox_guggy_button', 'cmd' => 'chatbox_guggy_cmd', 'both' => 'chatbox_guggy_both' ) ), NULL, NULL, NULL, 'chatbox_guggy' ) );	
		
		# System messages
		$form->addTab('chatbox_tab_auto_msg');
		$form->addHeader('chatbox_header_sysMSG');
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_announcement_on', $opts['announcement_on'] ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_online_on', $opts['online_on'] ) );
		
		# Permissions
		$form->addTab( 'room_tab_permissions' );
		$groups = \IPS\Member\Group::groups();	
		$form->add( new \IPS\Helpers\Form\Select( 'room_clubs_can_see_online', array_filter( explode( ',', \IPS\Settings::i()->room_clubs_can_see_online ) ), FALSE, array( 'options' => $groups, 'parse' => 'normal', 'multiple' => TRUE ) ) );
		
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			foreach( \array_keys( $values ) as $key )
			{
				if ( mb_strpos( $key, 'chatbox_' ) !== FALSE )
				{
					$dataOpts[ str_replace( 'chatbox_', '', $key ) ] = $values[ $key ];
					unset( $values[ $key ] );
				}
			}

			if ( \count( $dataOpts ) > 0 )
			{
				$values['room_clubs_configs'] = json_encode( $dataOpts );
			}	

			$form->saveAsSettings( $values );
			
			\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
		}
		
		/* Output */
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=chatbox&module=chatbox&controller=settings" ), 'chatbox_settings' );
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=chatbox&module=chatbox&controller=clubs" ), 'menu__chatbox_chatbox_clubs' );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('menu__chatbox_chatbox_clubs');
		\IPS\Output::i()->output = $form;
	}
	
	// Create new methods with the same name as the 'do' parameter which should execute it
}