<?php


namespace IPS\chatbox\modules\admin\chatbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * settings
 */
class _settings extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'skins.css', 'chatbox', 'front' ) );
		
		if ( \IPS\Request::i()->do != 'notifications' )
		{
			\IPS\Output::i()->sidebar['actions'] = array();

			\IPS\Output::i()->sidebar['actions']['notifications'] = array(
				'icon'	=> 'bullhorn',
				'link'	=> \IPS\Http\Url::internal( 'app=chatbox&module=chatbox&controller=settings&do=notifications' ),
				'title'	=> 'menu__chatbox_chatbox_notifications',
			);			
		}
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		# Build Form
		$form = new \IPS\Helpers\Form;	
		$form->addTab( 'chatbox_tab_general' );
		$form->addHeader( 'chatbox_tab_general' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_sound_disable', \IPS\Settings::i()->chatbox_sound_disable, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_report', \IPS\Settings::i()->chatbox_report, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_ignore', \IPS\Settings::i()->chatbox_ignore, FALSE, array() ) );
		
		$form->addHeader( 'chatbox_tab_time' );
		$form->add( new \IPS\Helpers\Form\Text( 'chatbox_time_format', \IPS\Settings::i()->chatbox_time_format, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Select( 'chatbox_time_moment_locale', \IPS\Settings::i()->chatbox_time_moment_locale, TRUE, array(
			'options'	=> array(
				'af'			=> 'Afrikaans',
				'sq'			=> 'Albanian',
				'ar'			=> 'Arabic',
				'ar-dz'			=> 'Arabic (Algeria)',
				'ar-kw'			=> 'Arabic (Kuwait)',
				'ar-ly'			=> 'Arabic (Lybia)',
				'ar-ma'			=> 'Arabic (Morocco)',
				'ar-sa'			=> 'Arabic (Saudi Arabia)',
				'ar-tn'			=> 'Arabic (Tunisia)',
				'hy-am'			=> 'Armenian',
				'az'			=> 'Azerbaijani',
				'bm'			=> 'Bambara',
				'eu'			=> 'Basque',
				'be'			=> 'Belarusian',
				'bn'			=> 'Bengali',
				'bs'			=> 'Bosnian',
				'br'			=> 'Breton',
				'bg'			=> 'Bulgarian',
				'my'			=> 'Burmese',
				'km'			=> 'Cambodian',
				'ca'			=> 'Catalan',
				'tzm'			=> 'Central Atlas Tamazight',
				'tzm-latn'		=> 'Central Atlas Tamazight Latin',
				'zh-cn'			=> 'Chinese (China)',
				'zh-hk'			=> 'Chinese (Hong Kong)',
				'zh-tw'			=> 'Chinese (Taiwan)',
				'cv'			=> 'Chuvash',
				'hr'			=> 'Croatian',
				'cs'			=> 'Czech',
				'da'			=> 'Danish',
				'nl'			=> 'Dutch',
				'nl-be'			=> 'Dutch (Belgium)',
				'en-au'			=> 'English (Australia)',
				'en-ca'			=> 'English (Canada)',
				'en-ie'			=> 'English (Ireland)',
				'en-il'			=> 'English (Israel)',
				'en-nz'			=> 'English (New Zealand)',
				'en-SG'			=> 'English (Singapore)',
				'en-gb'			=> 'English (United Kingdom)',
				'en'			=> 'English (United States)',
				'eo'			=> 'Esperanto',
				'et'			=> 'Estonian',
				'fo'			=> 'Faroese',
				'fi'			=> 'Finnish',
				'fr'			=> 'French',
				'fr-ca'			=> 'French (Canada)',
				'fr-ch'			=> 'French (Switzerland)',
				'fy'			=> 'Frisian',
				'gl'			=> 'Galician',
				'ka'			=> 'Georgian',
				'de'			=> 'German',
				'de-at'			=> 'German (Austria)',
				'de-ch'			=> 'German (Switzerland)',
				'el'			=> 'Greek',
				'gu'			=> 'Gujarati',
				'he'			=> 'Hebrew',
				'hi'			=> 'Hindi',
				'hu'			=> 'Hungarian',
				'is'			=> 'Icelandic',
				'id'			=> 'Indonesian',
				'ga'			=> 'Irish or Irish Gaelic',
				'it'			=> 'Italian',
				'it-ch'			=> 'Italian (Switzerland)',
				'ja'			=> 'Japanese',
				'jv'			=> 'Javanese',
				'kn'			=> 'Kannada',
				'kk'			=> 'Kazakh',
				'tlh'			=> 'Klingon',
				'gom-latn'		=> 'Konkani Latin script',
				'ko'			=> 'Korean',
				'ku'			=> 'Kurdish',
				'ky'			=> 'Kyrgyz',
				'lo'			=> 'Lao',
				'lv'			=> 'Latvian',
				'lt'			=> 'Lithuanian',
				'lb'			=> 'Luxembourgish',
				'mk'			=> 'Macedonian',
				'ms-my'			=> 'Malay',
				'ms'			=> 'Malay',
				'ml'			=> 'Malayalam',
				'dv'			=> 'Maldivian',
				'mt'			=> 'Maltese (Malta)',
				'mi'			=> 'Maori',
				'mr'			=> 'Marathi',
				'mn'			=> 'Mongolian',
				'me'			=> 'Montenegrin',
				'ne'			=> 'Nepalese',
				'se'			=> 'Northern Sami',
				'nb'			=> 'Norwegian Bokmål',
				'nn'			=> 'Nynorsk',
				'fa'			=> 'Persian',
				'pl'			=> 'Polish',
				'pt'			=> 'Portuguese',
				'pt-br'			=> 'Portuguese (Brazil)',
				'x-pseudo'		=> 'Pseudo',
				'pa-in'			=> 'Punjabi (India)',
				'ro'			=> 'Romanian',
				'ru'			=> 'Russian',
				'gd'			=> 'Scottish Gaelic',
				'sr'			=> 'Serbian',
				'sr-cyrl'		=> 'Serbian Cyrillic',
				'sd'			=> 'Sindhi',
				'si'			=> 'Sinhalese',
				'sk'			=> 'Slovak',
				'sl'			=> 'Slovenian',
				'es'			=> 'Spanish',
				'es-do'			=> 'Spanish (Dominican Republic)',
				'es-us'			=> 'Spanish (United States)',
				'sw'			=> 'Swahili',
				'sv'			=> 'Swedish',
				'tl-ph'			=> 'Tagalog (Philippines)',
				'tg'			=> 'Tajik',
				'tzl'			=> 'Talossan',
				'ta'			=> 'Tamil',
				'te'			=> 'Telugu',
				'tet'			=> 'Tetun Dili (East Timor)',
				'th'			=> 'Thai',
				'bo'			=> 'Tibetan',
				'tr'			=> 'Turkish',
				'uk'			=> 'Ukrainian',
				'ur'			=> 'Urdu',
				'ug-cn'			=> 'Uyghur (China)',
				'uz'			=> 'Uzbek',
				'uz-latn'		=> 'Uzbek Latin',
				'vi'			=> 'Vietnamese',
				'cy'			=> 'Welsh',
				'yo'			=> 'Yoruba Nigeria',
				'ss'			=> 'siSwati'
			)
		) ) );
		
		$form->addTab( 'chatbox_tab_bottom' );
		$form->addHeader( 'chatbox_header_bar_items' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_bar_room', \IPS\Settings::i()->chatbox_bar_room, FALSE, array(
			'togglesOn'	=> array( 'chatbox_bar_show_rooms' )
		) ) );
		
		$form->add( new \IPS\Helpers\Form\Node( 'chatbox_bar_show_rooms', explode(",", \IPS\Settings::i()->chatbox_bar_show_rooms), FALSE, array(
			'class' 			=> 'IPS\chatbox\Rooms\Room', 
			'multiple' 			=> TRUE
		), NULL, NULL, NULL, 'chatbox_bar_show_rooms' ) );		
		
		$form->add( new \IPS\Helpers\Form\Custom( 'chatbox_bar_setup_con', NULL, FALSE, array( 'getHtml' => function( $element )
		{
			return \IPS\Member::loggedIn()->language()->get( 'chatbox_bar_setup_con_help' );
		} ) ) );
			
		$form->addHeader( 'chatbox_header_bar' );
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'chatbox_bar_mini_devices', isset( \IPS\Settings::i()->chatbox_bar_mini_devices ) ? explode(',', \IPS\Settings::i()->chatbox_bar_mini_devices) : array( 'Phone', 'Tablet', 'Desktop' ), FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'chatbox_bar_mini_devices' ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'chatbox_bar_hide_mobile', \IPS\Settings::i()->chatbox_bar_hide_mobile, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\TextArea( 'chatbox_bar_expages', \IPS\Settings::i()->chatbox_bar_expages, FALSE, array( 'rows' => 8 ) ) );
		
		
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			unset( $values['chatbox_bar_setup_con'] );
			$form->saveAsSettings( $values );
		}
		
		/* Output */
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=chatbox&module=chatbox&controller=settings" ), 'chatbox_settings' );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('chatbox_settings');	
		\IPS\Output::i()->output .= $form;
	}
	
	/**
	 * notifications settings
	 */
	protected function notifications()
	{
		\IPS\Output::i()->output .= \IPS\Member::loggedIn()->language()->addToStack('chatbox_ntf_help');
		
		$opts = json_decode( \IPS\Settings::i()->chatbox_notifications, TRUE );

		$lang = \IPS\Member::loggedIn()->language();
		
		# Build Form
		$form = new \IPS\Helpers\Form;		
		
		$form->addTab( 'chatbox_notifications_in_rooms' );
		$form->add( new \IPS\Helpers\Form\Node( 'rooms', isset( $opts['rooms'] ) ? $opts['rooms'] : NULL, FALSE, array(
			'class' 			=> 'IPS\chatbox\Rooms\Room', 
			'multiple' 			=> TRUE
		) ) );
						
		foreach( \IPS\Application::allExtensions( 'core', 'ContentRouter' ) as $key => $router )
		{
			$addedTab = false;

			//if ( ! \in_array( $key, array( 'chatbox_Room', 'custompages_Pages', 'cms_Pages' ) ) )
			if ( \in_array( $key, array( 'core_Clubs', 'forums_Topics', 'gallery_gallery', 'cms_Records', 'calendar_calendar', 'blog_Blog', 'downloads_Files', 'videobox_Videobox', 'musicbox_Musicbox' ) ) )
			{
				foreach( $router->classes as $class )
				{
					$cnt++;
					
					if ( !$addedTab )
					{
						$form->addTab( '__app_' . $class::$application );
						$addedTab = true;
					}
					
					if( isset( $class::$containerNodeClass ) )
					{
						$containerClass = $class::$containerNodeClass;
						$containerName = str_replace( '\\', '_', $containerClass );
						$form->add( new \IPS\Helpers\Form\Node( $containerName, isset( $opts[ $containerName ] ) ? ( $opts[ $containerName ] == '0' ? 0 : $opts[ $containerName ] ) : NULL, FALSE, array(
							'class' 			=> $containerClass, 
							'zeroVal' 			=> 'all', 
							'multiple' 			=> TRUE
						), NULL, NULL, NULL, 'container_' . $cnt ) );
						$lang->words[ $containerName ] = $lang->checkKeyExists( $containerClass::$nodeTitle ) ? $lang->get( $containerClass::$nodeTitle ) : $containerClass::$nodeTitle;
					}
					
					$itemName = str_replace( '\\', '_', $class );
					$form->add( new \IPS\Helpers\Form\YesNo( $itemName, $opts[ $itemName ], FALSE ) );
					$lang->words[$itemName] = $lang->checkKeyExists( $class::$title . '_pl' ) ? $lang->get( $class::$title . '_pl' ) : $class::$title . '_pl';
					
					if ( isset( $class::$commentClass ) )
					{
						$commentClass = $class::$commentClass;
						$commentName = str_replace( '\\', '_', $commentClass );
						$form->add( new \IPS\Helpers\Form\YesNo( $commentName, $opts[ $commentName ], FALSE, array(), NULL, NULL, NULL, 'comment_' . $cnt ) );
						$lang->words[ $commentName ] = $lang->checkKeyExists( $commentClass::$title . '_pl' ) ? $lang->get( $commentClass::$title . '_pl' ) : $commentClass::$title . '_pl';
					}
					
					if ( isset( $class::$reviewClass ) )
					{
						$reviewClass = $class::$reviewClass;
						$reviewName = str_replace( '\\', '_', $reviewClass );
						$form->add( new \IPS\Helpers\Form\YesNo( $reviewName, $opts[ $reviewName ], FALSE, array(), NULL, NULL, NULL, 'review_' . $cnt ) );
						$lang->words[ $reviewName ] = $lang->checkKeyExists( $reviewClass::$title . '_pl' ) ? $lang->get( $reviewClass::$title . '_pl' ) : $reviewClass::$title . '_pl';
					}						
				}
			}
		}
	
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			foreach( \array_keys( $values ) as $key )
			{
				$dataOpts[ $key ] = $values[ $key ];
				unset( $values[ $key ] );
			}

			if ( \count( $dataOpts ) > 0 )
			{
				$values['chatbox_notifications'] = json_encode( $dataOpts );
			}	

			$form->saveAsSettings( $values );
		}
		
		/* Output */
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=chatbox&module=chatbox&controller=settings" ), 'chatbox_settings' );
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=chatbox&module=chatbox&controller=settings&do=notifications" ), 'menu__chatbox_chatbox_notifications' );		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('menu__chatbox_chatbox_notifications');
		\IPS\Output::i()->output .= $form;		
	}	 
	
}