<?php


namespace IPS\chatbox\modules\front\chatbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * room
 */
class _room extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		# Load room
		try
		{
			$this->room = \IPS\chatbox\Rooms\Room::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2CHATBOX100/2', 404 );
		}

		parent::execute();		
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		# Set breadcrumb
		if ( $club = $this->room->club() )
		{
			\IPS\core\FrontNavigation::$clubTabActive = TRUE;
			\IPS\Output::i()->breadcrumb = array();
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=clubs&controller=directory', 'front', 'clubs_list' ), \IPS\Member::loggedIn()->language()->addToStack('module__core_clubs') );
			\IPS\Output::i()->breadcrumb[] = array( $club->url(), $club->name );
			\IPS\Output::i()->breadcrumb[] = array( $this->room->url(), $this->room->_title );
			
			if ( \IPS\Settings::i()->clubs_header == 'sidebar' )
			{
				\IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'clubs', 'core' )->header( $club, $this->room, 'sidebar' );
			}
		}
		else
		{
			\IPS\Output::i()->breadcrumb[] = array( NULL, $this->room->_title );
		}
		
		# Output
		\IPS\Output::i()->title	= $this->room->_title;
		
		if ( \IPS\Request::i()->full != '1' )
		{
			\IPS\Output::i()->output = $this->room->show();
		}
		else
		{
			\IPS\Output::i()->sendOutput( \IPS\Theme::i()->getTemplate( 'global', 'core' )->blankTemplate( $this->room->show() ) );
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Csrf & ajax check
	/*-------------------------------------------------------------------------*/	
	protected function ajaxCheck()
	{
		if ( ! \IPS\IN_DEV )
		{
			\IPS\Session::i()->csrfCheck();
			
			if ( ! \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->error( 'chatbox_error_noajax', '2CHATBOX100/3', 403, '' );
			}		
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Check view permission
	/*-------------------------------------------------------------------------*/		
	protected function checkViewPerm()
	{
		if ( ! $this->room->_canView() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_view' ) ) );
			exit;
		}
		if ( $this->room->isBlocked() )
		{
			\IPS\Output::i()->json( array( 'type' => 'isBlocked', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_kick_out' ) ) );
			exit;
		}	

		# Block spiders
		if( \IPS\Session\Front::i()->userAgent->spider )
		{
			\IPS\Output::i()->error( 'chatbox_error_no_perm_view', '2CHATBOX100/4', 403, '' );
		}		
	}

	/*-------------------------------------------------------------------------*/
	// Check chat permission
	/*-------------------------------------------------------------------------*/	
	protected function checkChatPerm()
	{
		if ( ! $this->room->_canChat() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_chat' ) ) );
			exit;
		}
		if ( $this->room->isBlocked() )
		{
			\IPS\Output::i()->json( array( 'type' => 'isBlocked', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_kick_out' ) ) );
			exit;
		}
	}	
	
	/*-------------------------------------------------------------------------*/
	// Ping to keep us in chatters list
	/*-------------------------------------------------------------------------*/	 
	protected function ping()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Check permission
		$this->checkViewPerm();

		# Chatters
		$chatters = $this->room->getChatters();

		# Update last ping
		$key = $this->room->userKey();
		
		if ( isset( $chatters[ $key ] ) )
		{
			# Update last Ping
			$chatters[ $key ]['last_ping'] = time();
			
			# Remove inactive users
			foreach( $chatters as $k => $user )
			{
				if ( $user['last_ping'] <= \IPS\DateTime::create()->sub( new \DateInterval( 'PT300S' ) )->getTimeStamp() )
				{
					if ( $this->room->options()['online_on'] )
					{
						$this->room->addSystemMsg( 'leave', $user['chatterName'] );
					}
					unset( $chatters[$k] );
				}
			}
			
			$this->room->saveChatters( $chatters );
		}
		else
		{
			$chatters = $this->addUser();
		}
		
		# Ouput
		\IPS\Output::i()->json('OK');	
	}
	
	/*-------------------------------------------------------------------------*/
	// add user to data list
	/*-------------------------------------------------------------------------*/	 
	protected function addUser()
	{
		# Get chatters from datastore
		$chatters = $this->room->getChatters();
		
		$key = $this->room->userKey();

		# Add user to datastore
		if ( !isset( $chatters[ $key ] ) )
		{
			$chatters[ $key ] = $this->room->userInfo();
			if ( $this->room->options()['online_on'] && \IPS\Request::i()->joined != 1 )
			{
				$this->room->addSystemMsg( 'join', $chatters[ $key ]['chatterName'] );
			}
			$this->room->saveChatters( $chatters );
		}
		
		return $chatters;
	}
	
	/*-------------------------------------------------------------------------*/
	// Check and parse message
	/*-------------------------------------------------------------------------*/	
	protected function checkMSG( $msg )
	{
		$opts = $this->room->options();
		
		# Clean the format
		if ( $opts['use_editor'] != 1 )
		{
			$msg = \IPS\Application::load('chatbox')->makeCleanText( $msg );
		}
		
		# Is empty?
		if ( !( isset( $msg ) && ( mb_strlen(trim($msg) ) > 0) ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_empty_msg' ) ) );
			exit;				
		}
		
		# Check length
		if ( $opts['length'] && mb_strlen( $msg ) > $opts['length'] )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => sprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_error_maxlength' ), $opts['length'] ) ) );
			exit;				
		}
		
		# Prevent flooding special chars
		$temp = \IPS\Application::load('chatbox')->replaceEmoji( $msg, false );

		// if ( preg_match( '/[^\w\s\-!@<>#:{}`;,,﻿‚“”‘’„—=\+\|\"\'\~\[\]\.\?\/\$%\^&\*\(\)]+/u', $temp ) )
		// {
			// \IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_badchars' ) ) );
			// exit;	
		// }
		
		$temp = preg_split('//u', $temp, null, PREG_SPLIT_NO_EMPTY);
		
		foreach( $temp as $a )
		{
			if ( preg_match( '/[^\w\s]+/u', $a ) )
			{
				$cnt++;
				if ( $cnt >= 7 )
				{
					\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_badchars' ) ) );
					exit;
				}
			}
			else
			{
				$cnt = 0;
			}
		}
		
		# Done
		return $msg;
	}

	/*-------------------------------------------------------------------------*/
	// Send message
	/*-------------------------------------------------------------------------*/	
	protected function sendMSG()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Check permission
		$this->checkChatPerm();

		# Check and parse msg
		$msg = $this->checkMSG( \IPS\Request::i()->msg );

		# Check user in room
		$chatters = $this->room->getChatters();
		
		$key = $this->room->userKey();
		
		if ( ! isset( $chatters[ $key ] ) )
		{
			$chatters = $this->addUser();
		}

		# Check Flood
		if ( $this->room->options()['flood'] > 0 && !$this->room->_canManage() )
		{
			if ( time() - $chatters[ $key ]['last_chat'] < $this->room->options()['flood'] )
			{
				$wait = $this->room->options()['flood'] - ( time() - $chatters[ $key ]['last_chat'] );
				\IPS\Output::i()->json( array( 'type' => 'error', 'message' => sprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_error_flood' ), $wait ) ) );
				exit;
			}
		}
				
		# Save message
		$title = \IPS\Application::load('chatbox')->autoTitle( $msg );
		$title_furl = \IPS\Http\Url\Friendly::seoTitle( $title );
		$id = \IPS\Db::i()->insert( 'chatbox_messages', array(
			'room'				=> $this->room->id,
			'member_id'			=> \IPS\Member::loggedIn()->member_id,
			'guestName'			=> \IPS\Request::i()->cookie['chatbox_guestName'],
			'content'			=> $msg,
			'title'				=> $title,
			'title_furl'		=> $title_furl,
			'time'				=> time(),
		) );
		
		# Cache latest ID
		$this->room->cacheLatestID( $id );
		
		# Save last time chat to datastore
		$chatters[ $key ]['last_chat'] = time();
		$this->room->saveChatters( $chatters );
		
		# Ouput		
		\IPS\Output::i()->json( 'OK' );
	}
	
	/*-------------------------------------------------------------------------*/
	// get msg by id
	/*-------------------------------------------------------------------------*/
	protected function getMsgByID( $id )
	{
		try
		{
			$chat = \IPS\chatbox\Rooms\Chat::load( $id );
			return $chat;
		}
		catch ( \UnderflowException $e ) 
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_nomsg' ) ) );
			exit;						
		}
	}
		
	/*-------------------------------------------------------------------------*/
	// Get New Messages
	/*-------------------------------------------------------------------------*/	
	protected function getMSG()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Check permission
		$this->checkViewPerm();
		
		# Options
		$useSQL = true;
		$loadMoreMode = (int) \IPS\Request::i()->loadMoreMode;
		$opts = $this->room->options();

		# Check lastID
		$lastID = \IPS\Request::i()->lastID ? (int) \IPS\Request::i()->lastID : 0;
		
		if ( $loadMoreMode == 0 && $lastID > 0 )
		{
			$cacheLastID = $this->room->cacheLatestID();
			if ( $cacheLastID > -1 && $cacheLastID <= $lastID ) // no new msg
			{
				$useSQL = false;
				$newLastID = $cacheLastID;
			}
		}

		$return['cacheLevel'] = $useSQL ? 0 : 1;
		
		# Query
		if ( $useSQL )
		{
			if( \IPS\Settings::i()->chatbox_ignore == 1 && \IPS\Member::loggedIn()->member_id )
			{
				$ignored = iterator_to_array( \IPS\Db::i()->select( 'ignore_ignore_id, ignore_id', 'core_ignored_users', array( 'ignore_owner_id=? and ignore_chatboxplus=1', \IPS\Member::loggedIn()->member_id ) )->setValueField( 'ignore_ignore_id' ) );
				if ( \count( $ignored ) > 0 )
				{
					$where[] = array( 'member_id NOT IN(' . implode( ",", $ignored ) . ')' );
				}			
			}

			$where[] = array( '( room=? OR FIND_IN_SET( ?, in_multiple_rooms ) )', $this->room->id, $this->room->id ); 
			
			if ( !$opts['announcement_on'] )
			{
				$where[] = array( '( system IS NULL OR system != ? )', 'announcement' );
			}
			
			if ( !$this->room->_canSeeOnline() || !$opts['online_on'] )
			{
				$where[] = array( '( system IS NULL OR ( system != ? AND system != ? ) )', 'join', 'leave' );
			}
			
			$where[] = $loadMoreMode == 1 ? array( 'id<?', $lastID ) : array( 'id>?', $lastID );
			
			$chats = iterator_to_array( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'chatbox_messages', $where, 'id DESC', array( 0, (int) $opts['limit'] ) ), 'IPS\chatbox\Rooms\Chat' ) );
			$total = \count( $chats );
			
			if ( $total > 0 )
			{
				$newLastID = $chats[0]->id;
				
				$chats = $opts['sort'] == 'asc' ? array_reverse($chats) : $chats;
				
				foreach( $chats as $chat )
				{
					if ( \intval( $chat->member_id ) > 0 )	// Msg from member
					{
						$mem = \IPS\Member::load( $chat->member_id );
						$chatterID = $mem->member_id;
						$chatterName = $mem->name;
						$chatterKey = "user@" . $mem->member_id;
						$chatterPhoto = $mem->photo;
						$chatterNameFormat = $mem->member_group_id ? \IPS\Member\Group::load( $mem->member_group_id )->formatName( $chatterName ) : $chatterName;						
						$content = $this->room->formatContent( $chat->content );
					}
					elseif ( \intval( $chat->member_id ) == 0 ) // Msg from guest
					{
						$chatterID = 0;
						$chatterName = $chat->guestName;
						$chatterKey = "guest@" . $chat->guestName;
						$chatterPhoto = md5( $chatterName );
						$chatterNameFormat = \IPS\Member\Group::load( \IPS\Settings::i()->guest_group )->formatName( $chatterName );
						$content = $this->room->formatContent( $chat->content );
					}
					else // Msg from system
					{
						$content 	= $chat->content;
						$chatterID 	= -1;
						$v 			= NULL;
						$g 			= NULL;
						$gPrefix 	= "";
						
						if ( $chat->system == 'content' )
						{
							$v = json_decode( $content, true );				
						}					
						if ( $chat->system == 'announcement' )
						{
							if ( $content == 'remove' )
							{
								$updateAnn = $content;
								$content = NULL;
							}
							else
							{
								$updateAnn = $this->room->announcement()[0];
								$v = array( (string) $this->room->url('readAnnouncement'), $updateAnn );
							}
						}
						if ( $chat->system == 'join' || $chat->system == 'leave' )
						{
							$user = \IPS\Member::load( $content, 'name' );

							if ( $user->member_group_id == \IPS\Settings::i()->guest_group )
							{
								$gPrefix = \IPS\Member::loggedIn()->language()->get( 'chatbox_guest_name' ) . " ";
							}

							$v = array( $gPrefix, \IPS\Theme::i()->getTemplate( 'room', 'chatbox', 'front' )->chatterName( \IPS\Member\Group::load( $user->member_group_id )->formatName( $content ), $user->member_id, (string) $user->url() ) );
						}
						
						$content = vsprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_sys_' . $chat->system ), $v );
					}
					
					if ( $content )
					{
						$moreThanOneDay = ( time() - $chat->time ) <= 86400 ? true : false;
						
						$new[] = array(
							'id'				=> $chat->id,
							'chatterID'			=> $chatterID,
							'chatterName' 		=> $chatterName,
							'chatterKey' 		=> $chatterKey,
							'chatterPhoto' 		=> $chatterPhoto,
							'chatterNameFormat' => $chatterNameFormat,
							'chatterIP' 		=> $this->room->_canManage() ? \IPS\Request::i()->ipAddress() : NULL,
							'content' 			=> $content,
							'showAgoTime'		=> $moreThanOneDay ? $chat->time : false,
							'showDate'			=> !$moreThanOneDay ? \IPS\DateTime::ts( $chat->time )->format( \IPS\Settings::i()->chatbox_time_format ) : "",
							'canEdit'			=> $this->room->_canEdit( $chat->member_id ),
							'canDelete'			=> $this->room->_canDelete( $chat->member_id ),
							'canReport'			=> $chat->canReport() && \IPS\Settings::i()->chatbox_report == 1,
						);
					}
				}
			}
			else
			{
				$newLastID = $cacheLastID > 0 ? $cacheLastID : null;
			}
		}
		
		# Get Chatters
		$chatters = $this->room->getChatters();

		# Return
		$return['content'] = $new;
		$return['lastID'] = $newLastID;

		if ( $updateAnn )
		{
			$return['updateAnn'] = $updateAnn;
		}
		if ( $opts['showUserList'] == 1 && $this->room->_canSeeOnline() )
		{
			$return['chatters'] = $chatters;
		}		
		if ( $this->room->_canPlaySong() )
		{
			$return['song'] = $this->room->song;
		}

		# update edited/deleted msg
		if ( isset( $chatters[ $this->room->userKey() ][ 'update' ] ) )
		{
			$return['update'] = $chatters[ $this->room->userKey() ][ 'update' ];
			unset( $chatters[ $this->room->userKey() ][ 'update' ] );
			$this->room->saveChatters( $chatters );
		}

		\IPS\Output::i()->json( $return );	
	}
	
	/*-------------------------------------------------------------------------*/
	// Delete Message
	/*-------------------------------------------------------------------------*/		
	protected function deleteMSG()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Get msg by ID
		$chat = $this->getMsgByID( \IPS\Request::i()->chatID );	
		
		# Check permission
		if ( ! $this->room->_canDelete( $chat->member_id ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_del' ) ) );
			exit;			
		}		

		# Save to datastore to update for all online users
		$this->room->markMsg( $chat->id );	
		
		# Delete
		if ( $chat->room > 0 || $chat->in_multiple_rooms === (string) $this->room->id )
		{
			\IPS\Db::i()->delete( 'chatbox_messages', array( 'id=?', $chat->id ) );
		}
		else
		{
			$r = explode( ",", $chat->in_multiple_rooms );
			$rooms = array_diff( $r, [ $this->room->id ] );
			\IPS\Db::i()->update( 'chatbox_messages', array( 'in_multiple_rooms' => implode( ",", $rooms ) ), array( 'id=?', $chat->id ) );
		}
			
		# Done
		if ( \IPS\Request::i()->isAjax() )		
		{
			\IPS\Output::i()->json( 'OK' );
		}
		else
		{
			\IPS\Output::i()->redirect( $this->room->url() );
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Edit Message
	/*-------------------------------------------------------------------------*/		
	protected function editMSG()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Get msg by ID
		$chat = $this->getMsgByID( \IPS\Request::i()->chatID );			

		# Check permission
		if ( ! $this->room->_canEdit( $chat->member_id ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_edit' ) ) );
			exit;			
		}
		
		# Done
		if ( \IPS\Request::i()->isAjax() )		
		{
			\IPS\Output::i()->json( array( 'content' => $chat->content ) );
		}
		else
		{
			\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
		}			
	}

	protected function doEdit()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Get msg by ID
		$chat = $this->getMsgByID( \IPS\Request::i()->chatID );			

		# Check permission
		if ( ! $this->room->_canEdit( $chat->member_id ) || $chat->system )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_edit' ) ) );
			exit;			
		}
						
		# Check and parse msg
		$msg = $this->checkMSG( \IPS\Request::i()->msg );
		
		# Save
		$chat->title = \IPS\Application::load('chatbox')->autoTitle( $msg );
		$chat->title_furl = \IPS\Http\Url\Friendly::seoTitle( $chat->title );
		$chat->content = $msg;
		$chat->save();

		# Save to datastore to update for all online users
		$this->room->markMsg( $chat->id, $msg );
		
		# Done
		if ( \IPS\Request::i()->isAjax() )		
		{
			\IPS\Output::i()->json( array( 'content' => $this->room->formatContent( $msg ) ) );	
		}
		else
		{
			\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
		}					
	}
		
	/*-------------------------------------------------------------------------*/
	// Delete all messages
	/*-------------------------------------------------------------------------*/			
	protected function cleanRoom()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_clean' ) ) );
			exit;			
		}

		# Update new message
		$this->room->cleanRoom();

		# Done
		\IPS\Output::i()->json( 'OK' );	
	}
	
	/*-------------------------------------------------------------------------*/
	// Announcement
	/*-------------------------------------------------------------------------*/			
	protected function makeAnnouncement()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_ann' ) ) );
			exit;			
		}

		# Form
		$form = new \IPS\Helpers\Form();
		$form->class = 'ipsForm_noLabels chatboxAnnForm';
		$form->add( new \IPS\Helpers\Form\Translatable( 'room_announcement_title', NULL, FALSE, array( 'app' => 'chatbox', 'key' => "chatbox_room_{$this->room->id}_announcement_title" ) ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'room_announcement', NULL, FALSE, array( 'app' => 'chatbox', 'key' => "chatbox_room_{$this->room->id}_announcement", 'editor' => array( 'app' => 'chatbox', 'key' => 'Room', 'autoSaveKey' => "chatbox-room-{$this->room->id}-announcement", 'attachIds' => array( $this->room->id, NULL, 'chatbox_announcement' ) ) ) ) );

		if ( $values = $form->values() )
		{
			if ( $values['room_announcement_title'] )
			{
				\IPS\Lang::saveCustom( 'chatbox', "chatbox_room_{$this->room->id}_announcement_title", $values['room_announcement_title'] );
			}
			else
			{
				\IPS\Lang::deleteCustom( 'chatbox', "chatbox_room_{$this->room->id}_announcement_title" );
			}
			
			if ( $values['room_announcement'] )
			{
				\IPS\Lang::saveCustom( 'chatbox', "chatbox_room_{$this->room->id}_announcement", $values['room_announcement'] );
			}
			else
			{
				\IPS\Lang::deleteCustom( 'chatbox', "chatbox_room_{$this->room->id}_announcement" );
			}
			
			$opts = $this->room->options();
			
			if ( $opts['announcement_on'] && isset( $values['room_announcement_title'] ) && $values['room_announcement'] )
			{
				$msgType = "announcement";
				\IPS\Db::i()->delete( 'chatbox_messages', array( 'system=? AND room=?', $msgType, $this->room->id ) );
				$this->room->addSystemMsg( $msgType );
			}
			
			\IPS\Output::i()->json( array(
				'message' 	=> \IPS\Member::loggedIn()->language()->addToStack( 'chatbox_updated_announcement' ),
				'title'		=> $this->room->announcement()[0],
			) );
			
			return;
		}		

		# Done
		\IPS\Output::i()->output = $form->customTemplate( array( \call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
	}
	
	protected function readAnnouncement()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Check permission
		$this->checkViewPerm();
		
		# Get announcement
		$ann = $this->room->announcement();

		if ( !$ann[0] )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_ann' ) ) );
			exit;			
		}
		
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'room', 'chatbox', 'front' )->announcement( $ann[1] );
	}
	
	/*-------------------------------------------------------------------------*/
	// Blocked list
	/*-------------------------------------------------------------------------*/	
	protected function blockedChatters()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_block' ) ) );
			exit;			
		}
		
		# List
		$data = array();
		
		if ( $this->room->blocked )
		{
			$data = json_decode( $this->room->blocked, true );
		}

		# Done
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'room', 'chatbox', 'front' )->blockedChatters( $data );
	}
	
	/*-------------------------------------------------------------------------*/
	// Block a user
	/*-------------------------------------------------------------------------*/	
	protected function addToBlockedList()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_block' ) ) );
			exit;			
		}
		
		# Key & name & ip
		$key 	= \IPS\Request::i()->key;
		$name 	= \IPS\Request::i()->name;
		$ip 	= \IPS\Request::i()->ip;
		
		# Delete all messages
		if ( \IPS\Request::i()->del == 1 )
		{
			\IPS\Task::queue( 'chatbox', 'deleteMsg', array( 'room' => $this->room->id, 'user' => $key ), 4 );
		}
		
		# Is blocked?
		if ( $this->room->blocked )
		{
			$data = json_decode( $this->room->blocked, true );
	
			if ( \in_array( $key, array_keys($data) ) )
			{
				\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_in_blocked' ) ) );
				exit;					
			}
		}

		# Block
		$data[ $key ] = array( 'chatterName' => $name, 'ip' => $ip );
		
		# Save
		$this->room->blocked = json_encode( $data );
		$this->room->save();
		
		# Remove from online list
		$chatters = $this->room->getChatters();
		if ( \in_array( $key, array_keys( $chatters ) ) )
		{
			unset( $chatters[ $key ] );		
			$this->room->saveChatters( $chatters );
		}
		
		# Done
		\IPS\Output::i()->json( array(
			'message' 	=> sprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_blocked_done' ), $name ),
		) );
	}
	
	/*-------------------------------------------------------------------------*/
	// Unblock
	/*-------------------------------------------------------------------------*/	
	protected function unblock()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_block' ) ) );
			exit;			
		}

		# Unblock
		$data = array();
		
		if ( $this->room->blocked )
		{
			$data = json_decode( $this->room->blocked, true );
		}

		if ( !$data[ \IPS\Request::i()->key ] )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_user' ) ) );
			exit;				
		}
		
		unset( $data[ \IPS\Request::i()->key ] );
		$this->room->blocked = \count($data) > 0 ? json_encode( $data ) : null;
		$this->room->save();
		
		# Done
		\IPS\Output::i()->json('OK');
	}
	
	/*-------------------------------------------------------------------------*/
	// Get Chatters List
	/*-------------------------------------------------------------------------*/	
	protected function chattersList()
	{
		$this->ajaxCheck();
		
		$chatters = $this->room->getChatters();
		
		if ( !$this->room->_canManage() )
		{
			foreach( $chatters as $k => $u )
			{
				unset( $chatters[$k]['ip'] );
			}
		}

		\IPS\Output::i()->json( array( 'list' => $chatters, 'count' => \count( $chatters ) ) );
	}
	
	/*-------------------------------------------------------------------------*/
	// Change Name
	/*-------------------------------------------------------------------------*/		
	protected function changeName()
	{
		# Ajax Check
		$this->ajaxCheck();

		# Not guest?
		if ( \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_not_guest' ) ) );
			exit;			
		}

		# Check name and save to cookie
		$name = $this->room->checkName( \IPS\Request::i()->name );

		# Check permission
		$this->checkViewPerm();
		
		# Is bottom?
		$inGlobal = \IPS\Request::i()->inGlobal == 1 ? true : false;
		
		# Done
		\IPS\Output::i()->json( array( 
			'message'	=> sprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_welcome_guest' ), $name ),
			'html' 		=> $this->room->show(NULL, $inGlobal),
		) );
	}
	
	/*-------------------------------------------------------------------------*/
	// Use editor to send msg
	/*-------------------------------------------------------------------------*/		
	protected function postEditor()
	{
		$form = $this->room->editor();
		
		if ( $values = $form->values() )
		{
			# Ajax Check
			$this->ajaxCheck();
			
			# Check permission
			$this->checkChatPerm();

			# Check user in room
			$chatters = $this->room->getChatters();
			
			$key = $this->room->userKey();
			
			if ( ! isset( $chatters[ $key ] ) )
			{
				$chatters = $this->addUser();
			}

			# Check Flood
			if ( $this->room->options()['flood'] > 0 && !$this->room->_canManage() )
			{
				if ( time() - $chatters[ $key ]['last_chat'] < $this->room->options()['flood'] )
				{
					$wait = $this->room->options()['flood'] - ( time() - $chatters[ $key ]['last_chat'] );
					\IPS\Output::i()->json( array( 'type' => 'error', 'message' => sprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_error_flood' ), $wait ) ) );
					exit;
				}
			}
			
			# Check message
			$content = $values['chatMSG_' . $this->room->id];
			$this->checkMSG( $content );
			
			# Save message
			$values['chatbox_room'] = $this->room->id;
			$chat = \IPS\chatbox\Rooms\Chat::createFromForm( $values );
			
			# Save last time chat to datastore
			$chatters[ $key ]['last_chat'] = time();
			$this->room->saveChatters( $chatters );
			
			# Cache latest ID
			$this->room->cacheLatestID( $chat->id );			
			
			# Ouput		
			\IPS\Output::i()->json( 'OK' );
		}
		
		\IPS\Output::i()->output = $form;		
	}
	
	/*-------------------------------------------------------------------------*/
	// Use editor to edit msg
	/*-------------------------------------------------------------------------*/	
	protected function editEditor()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Get msg by ID
		$chat = $this->getMsgByID( \IPS\Request::i()->chatID );			

		# Check permission
		if ( ! $this->room->_canEdit( $chat->member_id ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_edit' ) ) );
			exit;			
		}
		
		# Form
		$form = $this->room->editor($chat);
		
		if ( $values = $form->values() )
		{
			# Check message
			$content = $values['chatMSG_' . $this->room->id . '_' . $chat->id ];
			$this->checkMSG( $content );

			$values['chatMSG_' . $this->room->id ] = $content;
			unset( $values['chatMSG_' . $this->room->id . '_' . $chat->id ] );				

			# Save	
			$chat->processForm( $values );
			$chat->save();
			$chat->processAfterEdit( $values );	
			
			# Save to datastore to update for all online users
			$this->room->markMsg( $chat->id, $content );
		
			if ( \IPS\Request::i()->isAjax() )		
			{
				\IPS\Output::i()->json( array( 'content' => $content ) );
			}
			else
			{
				\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
			}
		
			return;
		}
		
		# Done
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'global', 'chatbox', 'front' )->editor( $form, $this->room->options() );
	}	

	/*-------------------------------------------------------------------------*/
	// Play a song
	/*-------------------------------------------------------------------------*/	
	protected function selectSong()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() || ! $this->room->_canPlaySong() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_song' ) ) );
			exit;			
		}

		if ( $this->room->song )
		{
			$songData = json_decode( $this->room->song, TRUE );
			
			try
			{
				$song = \IPS\musicbox\Song::load( $songData['id'] );
				if ( $song->isPlaylist != 1 )
				{
					$playingSong = \IPS\Theme::i()->getTemplate( 'room', 'chatbox', 'front' )->songPlaying( $song );
				}
			}
			catch ( \OutOfRangeException $e ) 
			{
				$this->room->song = NULL;
				$this->room->save();
			}			
		}
		
		# Done
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'room', 'chatbox', 'front' )->songSearch( $this->room, $playingSong );
	}
	
	protected function searchSong()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() || ! $this->room->_canPlaySong() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_song' ) ) );
			exit;			
		}
		
		# Search
		$q = mb_strtolower( urldecode( \IPS\Request::i()->q ) );
		if ( !$q || mb_strlen( $q ) < 4 )
		{
			return '';
		}
		
		$perpage = 5;
		$page = isset( \IPS\Request::i()->page ) ? \intval( \IPS\Request::i()->page ) : 1;
		if( $page < 1 )
		{
			$page = 1;
		}
		
		$url = $this->room->url()->setQueryString( array( 'do' => 'searchSong', 'q' => $q ) );
		
		$where[] = array( "(song_title LIKE CONCAT( '%', ?, '%' ) OR song_singer LIKE CONCAT( '%', ?, '%' ))", $q, $q );
		$where[] = array( 'song_isPlaylist=0' );				
		$select = \IPS\Db::i()->select( '*', 'musicbox_songs', $where, 'song_id DESC', array( ( $page - 1 ) * $perpage, $perpage ), NULL, NULL, \IPS\Db::SELECT_SQL_CALC_FOUND_ROWS );

		$pagination = \IPS\Theme::i()->getTemplate( 'global', 'core', 'global' )->pagination(
			$url,
			ceil( $select->count( TRUE ) / $perpage ),
			$page,
			$perpage,
			TRUE
		);
		
		foreach( $select as $row )
		{
			$data[] = \IPS\musicbox\Song::constructFromData( $row );
		}

		# Output
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('mb_search_results');
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'room', 'chatbox', 'front' )->songResults( $data, $pagination );
	}
	
	protected function addSong()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() || ! $this->room->_canPlaySong() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_song' ) ) );
			exit;			
		}

		# Add song
		try
		{
			$song = \IPS\musicbox\Song::load( (int) \IPS\Request::i()->song );

			if ( $song->isPlaylist != 1 )
			{
				$data = array(
					'file'		=> $song->musicFile(),
					'id'		=> $song->id,
					'url'		=> (string) $song->url(),
					'name'		=> $song->title,
					'artist'	=> $song->singer,
					'thumbnail'	=> $song->thumbnail(),
				);
				$songData = json_encode( $data );
				$this->room->song = $songData;
				$this->room->save();
		
				\IPS\Output::i()->json( array( 
					'html' 		=> \IPS\Theme::i()->getTemplate( 'room', 'chatbox', 'front' )->songPlaying( $song ), 
					'song' 		=> $songData, 
					'message' 	=> \IPS\Member::loggedIn()->language()->get( 'chatbox_added_song' ) 
				) );
				exit;
			}
		}
		catch ( \OutOfRangeException $e ) {}

		\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_song' ) ) );
	}
	
	protected function removeSong()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check permission
		if ( ! $this->room->_canManage() || ! $this->room->_canPlaySong() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_song' ) ) );
			exit;			
		}

		# Remove song
		$this->room->song = NULL;
		$this->room->save();
		
		\IPS\Output::i()->json( 'OK' );
	}	

	/*-------------------------------------------------------------------------*/
	// Change room
	/*-------------------------------------------------------------------------*/	
	protected function changeRoom()
	{
		# Ajax Check
		$this->ajaxCheck();

		# Load room
		try
		{
			$room = \IPS\chatbox\Rooms\Room::load( \IPS\Request::i()->newroom );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_room' ) ) );
			exit;	
		}
		
		if ( \IPS\Request::i()->inGlobal == 1 || \IPS\Request::i()->inWidget )
		{
			if ( \IPS\Request::i()->inGlobal == 1 )
			{
				$inGlobal = TRUE;
				\IPS\Request::i()->setCookie( 'chatbox_inRoom', $room->id, NULL, FALSE );
			}
			if ( \IPS\Request::i()->inWidget )
			{
				$widgetConf = array( 'orientation' => \IPS\Request::i()->inWidget );
			}
			
			\IPS\Output::i()->json( array( 'html' => $room->show( $widgetConf, $inGlobal ), 'config' => $room->cbVars() ) );
		}
		else
		{
			\IPS\Output::i()->json( array( 'url' => (string) $room->url() ) );
		}
	}
	
	
	
	
	
	
	
	
	
		
}