<?php


namespace IPS\chatbox\modules\front\conversation;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * main
 */
class _main extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		// This is the default method if no 'do' parameter is specified
	}
	
	/**
	 * Open a room
	 */
	protected function openRoom()
	{
		$this->ajaxCheck();
		
		try
		{
			$room = \IPS\chatbox\Rooms\Room::load( \IPS\Request::i()->room );
			
			if ( !$room->_canView() )
			{
				\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_room' ) ) );
				exit;				
			}
		}
		catch( \OutOfRangeException $e ) 
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_room' ) ) );
			exit;	
		}
	
		\IPS\Output::i()->json( array( 'html' => (string) $room->show(NULL, TRUE, TRUE), 'config' => $room->cbVars() ) );
	}

	/*-------------------------------------------------------------------------*/
	// Get conversation
	/*-------------------------------------------------------------------------*/
	protected function getCon( $id )
	{
		try
		{
			$con = \IPS\chatbox\Member\Conversation::load( $id );
			return $con;
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2CHATBOX103/1', 404 );
		}		
	}
	
	/*-------------------------------------------------------------------------*/
	// get msg by id
	/*-------------------------------------------------------------------------*/
	protected function getMsgByID( $id )
	{
		try
		{
			$chat = \IPS\chatbox\Member\Post::load( $id );
			return $chat;
		}
		catch ( \OutOfRangeException $e ) 
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_nomsg' ) ) );
			exit;						
		}
	}

	/*-------------------------------------------------------------------------*/
	// Update chats
	/*-------------------------------------------------------------------------*/	
	protected function updateChat()
	{
		# Check perm
		$this->ajaxCheck();
		
		# Ping to check the conversations & rooms
		if ( \IPS\Request::i()->activeTab == 0 )
		{
			if ( \IPS\Settings::i()->chatbox_bar_room == 1 )
			{
				$return['ping']['rooms'] = $this->pingRooms();
			}
			if ( \IPS\Settings::i()->chatbox_bar_con == 1 && \IPS\chatbox\Member\Conversation::_canUse() )
			{
				$chats = $this->pingChats();
				$return['ping']['chats'] = $chats;
			}
		}
		
		# Update new content in open tabs
		if ( \IPS\chatbox\Member\Conversation::_canUse() )
		{
			$return['tabs'] = $this->checkTabs(); 
		}
		
		# Mark as read
		if ( \IPS\Request::i()->focusTabs != 0 )
		{
			$readWhere = array( 'map_con_id IN (?) AND map_user_id=?', \IPS\Request::i()->focusTabs, \IPS\Member::loggedIn()->member_id );
			\IPS\Db::i()->update( 'chatbox_conversations_user_map', "map_unread=0, map_update=" . time(), $readWhere );										
		}	
			
		# Update edited/deleted msg
		if ( isset( \IPS\Data\Store::i()->chatbox_con_update ) )
		{
			$conCache = \IPS\Data\Store::i()->chatbox_con_update;
			if ( $conCache[ \IPS\Member::loggedIn()->member_id ] )
			{
				$return['changes'] = $conCache[ \IPS\Member::loggedIn()->member_id ];
			}
			unset( $conCache[ \IPS\Member::loggedIn()->member_id ] );
			\IPS\Data\Store::i()->chatbox_con_update = $conCache;
		}

		# Return
		\IPS\Output::i()->json( $return );
	}
	
	/*-------------------------------------------------------------------------*/
	// Ping
	/*-------------------------------------------------------------------------*/	
	protected function pingRooms()
	{	
		if ( !isset( \IPS\Data\Store::i()->chatbox_rooms_update ) )
		{
			\IPS\Data\Store::i()->chatbox_rooms_update = time();
		}
		
		if ( \IPS\Request::i()->roomsLastUpdate < \IPS\Data\Store::i()->chatbox_rooms_update )
		{
			foreach( \IPS\chatbox\Rooms\Room::availableRooms() as $room )
			{
				$rooms[] = array(
					'id'			=> $room->id,
					'title'			=> $room->_title,
					'icon'			=> $room->icon ? $room->icon : 'comments',
					'users'			=> $room->_canSeeOnline() ? $room->online() : null,
				);
			}

			$rooms[0]['lastUpdate'] = \IPS\Data\Store::i()->chatbox_rooms_update;
		}
		
		return $rooms;
	}
	
	protected function pingChats()
	{
		$where[] = array( 'm.map_user_id=? AND m.map_online=?', \IPS\Member::loggedIn()->member_id, 1 );
		
		$select = \IPS\Db::i()->select( '*', array( 'chatbox_conversations_user_map', 'm' ), $where, 'm.map_update DESC' )
			->join( array('chatbox_conversations', 'c'), 'c.con_id=m.map_con_id' )
			->join( array('chatbox_conversations_posts', 'p'), 'p.id=c.con_lastID' );
		
		foreach( $select as $row )
		{
			foreach( $row as $key => $val )
			{
				if ( mb_strpos( $key, 'con_' ) === 0 )
				{
					$conData[$key] = $val;
				}
			}

			$con = \IPS\chatbox\Member\Conversation::constructFromData( $conData );
			
			$members = $con->participants(); unset( $members[ \IPS\Member::loggedIn()->member_id ] );

			if ( \count( $members ) == 1 )
			{
				$receiver = reset( $members );
				$icon = $receiver->photo;
				$title = \IPS\Member\Group::load( $receiver->member_group_id )->formatName( $receiver->name );
			}
			else
			{
				foreach ( $members as $mid => $member )
				{
					$names[] = $member->name;
				}
				$title = implode(', ', $names);
			}
			
			$chats[] = array(
				'id'			=> $row['con_id'],
				'title'			=> $title,
				'icon'			=> $icon,
				'lastMsg'		=> \IPS\Application::load('chatbox')->makeCleanText( $row['content'] ),
				'lastMsgTime'	=> $row['time'] ? \IPS\DateTime::ts( $row['time'] )->relative() : null,
				'lastMsgID'		=> $con->lastID,
				'update'		=> $row['map_update'],
				'unread'		=> $row['map_unread'],
			);
		}
		
		if ( \count( $chats ) > 0 )
		{
			usort($chats, function ($item1, $item2) {
				return $item2['lastMsgID'] <=> $item1['lastMsgID'];
			});	
		}
		
		if ( (int) \IPS\Request::i()->conLastMsgID != $chats[0]['lastMsgID'] )
		{
			return $chats;		
		}
		
		return NULL;
	}
	
	/*-------------------------------------------------------------------------*/
	// Get all new messages in open tabs
	/*-------------------------------------------------------------------------*/
	protected function checkTabs()
	{
		# Options
		$opts = \IPS\chatbox\Member\Conversation::options();
		$loadMoreMode = (int) \IPS\Request::i()->loadMoreMode;
		
		# Get messages
		$tabs = json_decode( \IPS\Request::i()->tabData, TRUE );
		$totalTabs = \count( $tabs );

		if ( $totalTabs > 0 )
		{	
			foreach( $tabs as $k => $conf )
			{
				$conID = str_replace( 'convo_', '', $k );
				if ( \IPS\Request::i()->activeTab > 0 && \IPS\Request::i()->activeTab != $conID )
				{
					continue;
				}

				# Check lastID
				$lastID = $conf['lastID'] ? (int) $conf['lastID'] : 0;
				$useSQL = true;		
				if ( $loadMoreMode == 0 && $lastID > 0 )
				{
					$cacheLastID = \IPS\chatbox\Member\Conversation::cacheLatestID( $conID );
					if ( $cacheLastID > -1 && $cacheLastID <= $lastID ) // no new msg
					{
						$useSQL = false;
						$newLastID = $cacheLastID;
					}
				}

				$new[ $conID ]['cacheLevel'] = $useSQL ? 0 : 1;
				
				if ( $useSQL )
				{
					# Get all msg
					$where[ $conID ][] = $loadMoreMode > 0 ? array( 'id<?', $loadMoreMode ) : array( 'id>?', (int) $lastID );
					$where[ $conID ][] = array( 'con=?', $conID );
					$chats[ $conID ] = iterator_to_array( new \IPS\Patterns\ActiveRecordIterator( 
						\IPS\Db::i()->select( 
							'*', 
							'chatbox_conversations_posts', 
							$where[ $conID ], 
							'id DESC', 
							array( 0, (int) $opts['limit'] )
						),
						'IPS\chatbox\Member\Post'
					) );
				}
			}
			
			if ( \count( $chats ) > 0 )
			{
				foreach( $chats as $con => $posts )
				{	
					if ( \count( $posts ) == 0 )
					{
						continue;
					}
					
					$new[ $con ]['lastID'] = $posts[0]->id;
					
					$posts = $opts['sort'] == 'asc' ? array_reverse($posts) : $posts;
							
					foreach( $posts as $chat )
					{
						$content = $chat->content;
						
						if ( $opts['use_editor'] != 1 && \IPS\Settings::i()->emoji_style == 'twemoji' )
						{
							$content = \IPS\Application::load('chatbox')->replaceEmoji( $content );
						}
						
						if ( $content )
						{
							$mem = $chat->author();
							
							$moreThanOneDay = ( time() - $chat->time ) <= 86400 ? true : false;
							
							$new[ $con ]['messages'][] = array(
								'id'				=> $chat->id,
								'chatterID'			=> $mem->member_id,
								'chatterName' 		=> $mem->name,
								'chatterKey' 		=> "user@" . $mem->member_id,
								'chatterPhoto' 		=> $mem->photo,
								'chatterNameFormat' => $mem->member_group_id ? \IPS\Member\Group::load( $mem->member_group_id )->formatName( $mem->name ) : $mem->name,
								'content' 			=> $content,
								'showAgoTime'		=> $moreThanOneDay ? $chat->time : false,
								'showDate'			=> !$moreThanOneDay ? \IPS\DateTime::ts( $chat->time )->format( \IPS\Settings::i()->chatbox_time_format ) : "",
								'canEdit'			=> $chat->item()->_canEdit( $chat->member_id ),
								'canDelete'			=> $chat->item()->_canDelete( $chat->member_id ),
								'canReport'			=> FALSE,
							);
						}
					}				
				}
			}
		}

		if ( $loadMoreMode > 0 )
		{
			\IPS\Output::i()->json( array( 'tabs' => $new ) );
			exit;
		}
		
		return $new;
	}
	
	/*-------------------------------------------------------------------------*/
	// Check Use permission
	/*-------------------------------------------------------------------------*/	
	public function checkUsePerm()
	{
		if ( ! \IPS\chatbox\Member\Conversation::_canUse() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_chat' ) ) );
			exit;
		}
	}
	 
	/*-------------------------------------------------------------------------*/
	// Csrf & ajax check
	/*-------------------------------------------------------------------------*/	
	protected function ajaxCheck()
	{
		if ( ! \IPS\IN_DEV )
		{
			\IPS\Session::i()->csrfCheck();
			
			if ( ! \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->error( 'chatbox_error_noajax', '2CHATBOX100/3', 403, '' );
			}		
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Check and parse message
	/*-------------------------------------------------------------------------*/	
	protected function checkMSG( $msg )
	{
		$opts = \IPS\chatbox\Member\Conversation::options();
		
		# Clean the format
		if ( $opts['use_editor'] != 1 )
		{
			$msg = \IPS\Application::load('chatbox')->makeCleanText( $msg );
		}
		
		# Is empty?
		if ( !( isset( $msg ) && ( mb_strlen(trim($msg) ) > 0) ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_empty_msg' ) ) );
			exit;				
		}
		
		# Check length
		if ( $opts['length'] && mb_strlen( $msg ) > $opts['length'] )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => sprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_error_maxlength' ), $opts['length'] ) ) );
			exit;				
		}
		
		# Prevent flooding special chars
		$temp = \IPS\Application::load('chatbox')->replaceEmoji( $msg, false );
		$temp = preg_split('//u', $temp, null, PREG_SPLIT_NO_EMPTY);
		foreach( $temp as $a )
		{
			if ( preg_match( '/[^\w\s]+/u', $a ) )
			{
				$cnt++;
				if ( $cnt >= 7 )
				{
					\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_badchars' ) ) );
					exit;
				}
			}
			else
			{
				$cnt = 0;
			}
		}
		
		# Done
		return $msg;
	}	
	
	/*-------------------------------------------------------------------------*/
	// Send msg to
	/*-------------------------------------------------------------------------*/	
	public function openConversation()
	{
		$this->ajaxCheck();
		$this->checkUsePerm();
		
		if ( \IPS\Request::i()->conID )
		{
			$con = $this->getCon( \IPS\Request::i()->conID );
			if ( ! $con->_canChat() )
			{
				\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_chat' ) ) );
				exit;
			}			
		}
		else
		{
			# Check member ID
			$to = \IPS\Member::load( \IPS\Request::i()->memberID );
			
			if ( ! $to->member_id )
			{
				\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_user' ) ) );
				exit;	
			}
			
			if ( \IPS\Member::loggedIn()->member_id == $to->member_id )
			{
				\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_unknown' ) ) );
				exit;			
			}
			
			# Find the existing conversation
			$con = \IPS\chatbox\Member\Conversation::getExistingConvo( $to );
			
			# Update map
			\IPS\Db::i()->update( 'chatbox_conversations_user_map', "map_online=1", array( 'map_con_id=? AND map_user_id=?', $con->id, \IPS\Member::loggedIn()->member_id ) );
		}

		# Return
		\IPS\Output::i()->json( array( 'html' => (string) $con->show(), 'conID' => $con->id ) );
	}
	
	/*-------------------------------------------------------------------------*/
	// Send message
	/*-------------------------------------------------------------------------*/	
	protected function sendMSG()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Check use permission
		$this->checkUsePerm();

		# Check conversation
		$con = $this->getCon( \IPS\Request::i()->conID );
		if ( ! $con->_canChat() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_chat' ) ) );
			exit;
		}
		
		# Check and parse msg
		$msg = $this->checkMSG( \IPS\Request::i()->msg );
		
		# Options
		$opts = \IPS\chatbox\Member\Conversation::options();
		
		$lastChat = array();
		if ( $con->lastChat )
		{
			$lastChat = json_decode( $con->lastChat, TRUE );
		}
		
		# Check Flood
		if ( $opts['flood'] > 0 && !$con->_canManage() )
		{
			$myLastChat = (int) $lastChat[ \IPS\Member::loggedIn()->member_id ];
			
			if ( time() - $myLastChat < $opts['flood'] )
			{
				$wait = $opts['flood'] - ( time() - $myLastChat );
				\IPS\Output::i()->json( array( 'type' => 'error', 'message' => sprintf( \IPS\Member::loggedIn()->language()->get( 'chatbox_error_flood' ), $wait ) ) );
				exit;
			}
		}
				
		# Save message
		$title = \IPS\Application::load('chatbox')->autoTitle( $msg );
		$title_furl = \IPS\Http\Url\Friendly::seoTitle( $title );
		$id = \IPS\Db::i()->insert( 'chatbox_conversations_posts', array(
			'con'				=> $con->id,
			'member_id'			=> \IPS\Member::loggedIn()->member_id,
			'content'			=> $msg,
			'title'				=> $title,
			'title_furl'		=> $title_furl,
			'time'				=> time(),
		) );
		
		# Save last time chat & lastID
		$lastChat[ \IPS\Member::loggedIn()->member_id ] = time();
		$con->lastChat = json_encode( $lastChat );
		$con->lastID = $id;
		$con->save();
		
		# Count unread msg
		\IPS\Db::i()->update( 'chatbox_conversations_user_map', "map_unread=map_unread+1, map_online=1, map_update=" . time(), array( 'map_con_id=? AND map_user_id!=?', $con->id, \IPS\Member::loggedIn()->member_id ) );
		
		# Cache latest ID
		$con->cacheLatestID( $con->id, $id );	
			
		# Ouput		
		\IPS\Output::i()->json( 'OK' );
	}
	
	/*-------------------------------------------------------------------------*/
	// Delete the conversation
	/*-------------------------------------------------------------------------*/	
	protected function delCon()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Check use permission
		$this->checkUsePerm();	
		
		# Check conversation
		$con = $this->getCon( \IPS\Request::i()->conID );
		if ( ! $con->_canChat() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_del_con' ) ) );
			exit;
		}
		
		# Update conversation
		$con->lastID = 0;
		$con->lastChat = NULL;
		$con->save();
		
		# Delete all message
		\IPS\Task::queue( 'chatbox', 'deleteMsg', array( 'convo' => $con->id ), 4 );
		
		# Hide conversation map
		\IPS\Db::i()->update( 'chatbox_conversations_user_map', "map_unread=0, map_online=0", array( 'map_con_id=? AND map_user_id=?', $con->id, \IPS\Member::loggedIn()->member_id ) );

		# Done
		\IPS\Output::i()->json( 'OK' );		
	}
		
	/*-------------------------------------------------------------------------*/
	// Delete Message
	/*-------------------------------------------------------------------------*/		
	protected function deleteMSG()
	{
		# Ajax Check
		$this->ajaxCheck();
		
		# Check use permission
		$this->checkUsePerm();
		
		# Get msg by ID
		$chat = $this->getMsgByID( \IPS\Request::i()->chatID );	

		# Check permission
		if ( ! $chat->item()->_canDelete( $chat->member_id ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_del' ) ) );
			exit;			
		}		

		# Save to datastore to update for all online users
		$chat->item()->markMsg( $chat->id );	
		
		# Delete
		$chat->delete();
		if ( $chat->item()->lastID <= (int) \IPS\Request::i()->chatID )
		{
			$chat->item()->updateLastID();
		}
		
		# Done
		\IPS\Output::i()->json( 'OK' );
	}
	
	/*-------------------------------------------------------------------------*/
	// Edit Message
	/*-------------------------------------------------------------------------*/		
	protected function editMSG()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check use permission
		$this->checkUsePerm();
		
		# Get msg by ID
		$chat = $this->getMsgByID( \IPS\Request::i()->chatID );	

		# Check permission
		if ( ! $chat->item()->_canDelete( $chat->member_id ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_del' ) ) );
			exit;			
		}	
		
		# Done
		\IPS\Output::i()->json( array( 'content' => $chat->content ) );		
	}

	protected function doEdit()
	{
		# Ajax Check
		$this->ajaxCheck();
	
		# Check use permission
		$this->checkUsePerm();
			
		# Get msg by ID
		$chat = $this->getMsgByID( \IPS\Request::i()->chatID );			

		# Check permission
		if ( ! $chat->item()->_canEdit( $chat->member_id ) )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->get( 'chatbox_error_no_perm_edit' ) ) );
			exit;			
		}
						
		# Check and parse msg
		$msg = $this->checkMSG( \IPS\Request::i()->msg );
		
		# Save
		$chat->title = \IPS\Application::load('chatbox')->autoTitle( $msg );
		$chat->title_furl = \IPS\Http\Url\Friendly::seoTitle( $chat->title );
		$chat->content = $msg;
		$chat->save();

		# Save to datastore to update for all online users
		$chat->item()->markMsg( $chat->id, $msg );
		
		# Done
		$opts = \IPS\chatbox\Member\Conversation::options();
		if ( $opts['use_editor'] != 1 && \IPS\Settings::i()->emoji_style == 'twemoji' )
		{
			$msg = \IPS\Application::load('chatbox')->replaceEmoji( $msg );
		}
		\IPS\Output::i()->json( array( 'content' => $msg ) );					
	}		

	/*-------------------------------------------------------------------------*/
	// Find member
	/*-------------------------------------------------------------------------*/	
	protected function findMember()
	{
		# Check perm
		$this->ajaxCheck();
		$this->checkUsePerm();
		
		# Do search
		$results = array();
		
		$input = str_replace( array( '%', '_' ), array( '\%', '\_' ), mb_strtolower( \IPS\Request::i()->input ) );
		
		$where[] = array( "name LIKE CONCAT(?, '%')", $input );
		$where[] = array( "member_id != ?", \IPS\Member::loggedIn()->member_id );
		$where[] = array( \IPS\Db::i()->in( 'member_group_id', explode( ',', \IPS\Settings::i()->chatbox_con_can ), FALSE ) );
		
		$select = \IPS\Db::i()->select( '*', 'core_members', $where, 'LENGTH(name) ASC', array( 0, 20 ) );

		foreach ( $select as $row )
		{
			$member = \IPS\Member::constructFromData( $row );
			
			$results[] = array(
				'id'	=> 	$member->member_id,
				'value' => 	$member->name,
				'name'	=> 	htmlspecialchars( $member->name, ENT_DISALLOWED | ENT_QUOTES, 'UTF-8', FALSE ),
				'extra'	=> 	$member->groupName,
				'photo'	=> 	(string) $member->photo,
			);
		}
				
		\IPS\Output::i()->json( $results );
	}
	



}