<?php


namespace IPS\chatbox\Member;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Member Conversation Model
 */
class _Conversation extends \IPS\Content\Item
{
	/* !\IPS\Patterns\ActiveRecord */
	
	/**
	 * @brief	Database Table
	 */
	public static $databaseTable = 'chatbox_conversations';
	
	/**
	 * @brief	Database Prefix
	 */
	public static $databasePrefix = 'con_';
	
	/**
	 * @brief	Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[Content\Item]	Include the ability to search this content item in global site searches
	 */
	public static $includeInSiteSearch = FALSE;

	/**
	 * @brief	[Content\Comment]	Icon
	 */
	public static $icon = 'comments';
	
	/**
	 * Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		\IPS\Db::i()->delete( 'chatbox_conversations_user_map', array( 'map_con_id=?', $this->id ) );
		\IPS\Task::queue( 'chatbox', 'deleteMsg', array( 'convo' => $this->id ), 4 );
		
		parent::delete();
	}
	
	/* !\IPS\Content\Item */

	/**
	 * @brief	Title
	 */
	public static $title = 'chatbox_conversation';
	
	/**
	 * @brief	Database Column Map
	 */
	public static $databaseColumnMap = array(
		'title'				=> 'title',
		'date'				=> 'started_date',
		'author'			=> 'starter_id'
	);
	
	/**
	 * @brief	Application
	 */
	public static $application = 'chatbox';
	
	/**
	 * @brief	Module
	 */
	public static $module = 'conversation';
	
	/**
	 * @brief	Language prefix for forms
	 */
	public static $formLangPrefix = 'chatbox_con_';
	
	/**
	 * @brief	Comment Class
	 */
	public static $commentClass = 'IPS\chatbox\Member\Post';
	
	/**
	 * @brief	[Content\Item]	First "comment" is part of the item?
	 */
	public static $firstCommentRequired = TRUE;
	
	/**
	 * Should posting this increment the poster's post count?
	 *
	 * @param	\IPS\Node\Model|NULL	$container	Container
	 * @return	void
	 */
	public static function incrementPostCount( \IPS\Node\Model $container = NULL )
	{
		return FALSE;
	}
		
	/*-------------------------------------------------------------------------*/
	// Display a conversation
	/*-------------------------------------------------------------------------*/	
	public function show()
	{
		# Disable sound by default?
		if ( \IPS\Settings::i()->chatbox_sound_disable == 1 && !isset( \IPS\Request::i()->cookie[ 'chatbox_con_nosound_' . $this->id ] ) )
		{
			\IPS\Request::i()->setCookie( 'chatbox_con_nosound_' . $this->id, 1, NULL, FALSE );
		}	

		# Returl html
		return \IPS\Theme::i()->getTemplate( 'conversation', 'chatbox', 'front' )->conversation( $this, $this->options() );		
	}
	 
	 
	/**
	 * Can Merge?
	 *
	 * @param	\IPS\Member|NULL	$member	The member to check for (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canMerge( $member=NULL )
	{
		return FALSE;
	}

	/**
	 * Permissions
	 */
	public function canView( $member=NULL )
	{
		return $this->_canChat( $member );
	}

	public function _canUse( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();

		if ( ( $member->member_id > 0 && ( $member->restrict_post !== 0 || $member->temp_ban !== 0 ) ) || !$member->member_id )
		{
			return FALSE;
		}
		
		if ( $member->inGroup( explode( ",", \IPS\Settings::i()->chatbox_con_can ) ) )
		{
			return TRUE;		
		}
		
		return FALSE;		
	}	
	
	public function _canChat( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		
		if ( !$this->_canUse() )
		{
			return FALSE;
		}
		
		foreach ( $this->maps() as $map )
		{
			if ( $map['map_user_id'] === \IPS\Member::loggedIn()->member_id )
			{
				return TRUE;
			}
		}
		
		return FALSE;		
	}	

	public function _canEdit( $chatterID=0, $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		
		if ( !$this->_canChat() )
		{
			return FALSE;
		}

		if ( $chatterID > 0 && $member->member_id == $chatterID )
		{
			$opts = self::options();
			return $opts['bar_perm_edit'] == 1 ? TRUE : FALSE;	
		}
		
		return FALSE;		
	}
	
	public function _canDelete( $chatterID=0, $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		
		if ( !$this->_canChat() )
		{
			return FALSE;
		}

		if ( $chatterID > 0 && $member->member_id == $chatterID )
		{
			$opts = self::options();
			return $opts['bar_perm_del'] == 1 ? TRUE : FALSE;
		}
		
		return FALSE;		
	}		
	
	/**
	 * Can Manage?
	 */		
	public function _canManage( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		
		if ( $member->isAdmin() )
		{
			return TRUE;		
		}

		return FALSE;		
	}
	
	/**
	 * Can delete?
	 *
	 * @param	\IPS\Member|NULL	$member	The member to check for (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canDelete( $member=NULL )
	{
		if( \IPS\Member::loggedIn()->modPermission( 'can_view_reports' ) )
		{
			return TRUE;
		}
		
		return FALSE;
	}

	/**
	 * @brief	Cached URLs
	 */
	protected $_url	= array();

	/**
	 * Get URL
	 *
	 * @param	string|NULL		$action		Action
	 * @return	\IPS\Http\Url
	 */
	public function url( $action=NULL )
	{
		$_key	= md5( $action );

		if( !isset( $this->_url[ $_key ] ) )
		{
			$this->_url[ $_key ] = \IPS\Http\Url::internal( "app=chatbox&module=conversation&controller=conversation&id={$this->id}", 'front', 'chatbox_convo' );
		
			if ( $action )
			{
				$this->_url[ $_key ] = $this->_url[ $_key ]->setQueryString( 'do', $action );
			}
		}
	
		return $this->_url[ $_key ];
	}
	
	/**
	 * Get the map for the current member
	 *
	 * @return	mixed
	 */
	public function get_map()
	{
		$maps = $this->maps();
		
		if ( isset( $maps[ \IPS\Member::loggedIn()->member_id ] ) )
		{
			return $maps[ \IPS\Member::loggedIn()->member_id ];
		}
		
		throw new \OutOfRangeException;
	}
	
	
	/**
	 * @brief	Maps cache
	 */
	protected $maps = NULL;
	
	/**
	 * Get maps
	 */
	public function maps( $refresh = FALSE )
	{
		if ( $this->maps === NULL || $refresh === TRUE )
		{
			$this->maps = iterator_to_array( \IPS\Db::i()->select( '*', 'chatbox_conversations_user_map', array( 'map_con_id=?', $this->id ) )->setKeyField( 'map_user_id' ) );
		}
		return $this->maps;
	}

	/*-------------------------------------------------------------------------*/
	// Options
	/*-------------------------------------------------------------------------*/		
	protected static $_opts = NULL;
	
	public function options()
	{
		if ( static::$_opts !== NULL )
		{
			return static::$_opts;
		}
		
		$data = json_decode( \IPS\Settings::i()->chatbox_con_configs, true );
		
		if ( \count( $data ) == 0 )
		{
			$data = array (
				'style' 				=> 'standard',
				'skin' 					=> 'white',
				'height' 				=> 420,
				'sort' 					=> 'asc',
				'limit' 				=> 20,
				'interval' 				=> 10000,
				'flood' 				=> 5,
			);
		}

		static::$_opts = $data;
		return static::$_opts;		
	}
	
	public function cbVars()
	{
		$opts = self::options();

		$data = array(
			'interval' 		=> (int) $opts['interval'],
			'flood' 		=> (int) $opts['flood'],
			'limit' 		=> (int) $opts['limit'],		
			'length' 		=> (int) $opts['length'],
			'sort' 			=> $opts['sort'],
			'style' 		=> $opts['style'],
			'giphy' 		=> $opts['giphy'],	
			'guggy' 		=> $opts['guggy'],		
			'allow_img' 	=> $opts['allow_img'] ? true : false,
			'allow_video' 	=> $opts['allow_video'] ? true : false,		
			'autoload' 		=> $opts['autoload'] ? true : false,		
			'use_editor' 	=> $opts['use_editor'] ? true : false,
			'enter_editor' 	=> $opts['enter_editor'] ? true : false,
			'canManage' 	=> self::_canManage() ? true : false,
		);

		// Return js vars
		$vars[ 'chatbox_conversations' ] = $data;
		\IPS\Output::i()->endBodyCode .= \IPS\Theme::i()->getTemplate( 'embed', 'chatbox', 'front' )->chatvars( $vars );
	}
	
	/**
	 * Display conversations & rooms
	 */	
	public function showPopupCon()
	{
		if ( \IPS\Settings::i()->chatbox_bar_con != 1 || ! \IPS\chatbox\Member\Conversation::_canUse() )
		{
			return;
		}
		
		self::cbVars();
		$settings = self::options();
		$html .= self::restoreOpenedPopups();
		$html .= \IPS\Theme::i()->getTemplate( 'conversation', 'chatbox', 'front' )->main( $settings );

		return $html;
	}	
	
	/*-------------------------------------------------------------------------*/
	// Get participants
	/*-------------------------------------------------------------------------*/
	protected $participants = NULL;
	
	public function participants()
	{
		if( $this->participants !== NULL )
		{
			return $this->participants;
		}
		
		foreach( $this->maps() as $mid => $data )
		{
			$member = \IPS\Member::load( $mid );
			
			if ( !$member->member_id )
			{
				\IPS\Db::i()->delete( 'chatbox_conversations_user_map', array( 'map_con_id=? AND map_user_id=?', $this->id, $mid ) );
			}
			else
			{
				$this->participants[ $mid ] = $member;
			}
		}
		
		return $this->participants;
	}

	/*-------------------------------------------------------------------------*/
	// Restore opened popups
	/*-------------------------------------------------------------------------*/	 
	protected function restoreOpenedPopups()
	{
		$mid = \IPS\Member::loggedIn()->member_id;
		
		if ( $mid > 0 )
		{
			$cookieName = 'chatbox_mem_popups_' . $mid;
			
			if ( ! \IPS\Request::i()->cookie[ $cookieName ] )
			{
				return;
			}
			
			$popups = explode( "|", \IPS\Request::i()->cookie[ $cookieName ] );

			foreach( $popups as $k => $p )
			{
				$v = explode("_", $p);
				
				if ( $v[0] == "r" ) // room popups
				{
					try
					{
						$room = \IPS\chatbox\Rooms\Room::load( $v[1] );
						if ( ! $room->_canView() )
						{
							unset( $popups[$k] );
						}
						$html .= (string) $room->show(NULL, TRUE, TRUE);
					}
					catch( \OutOfRangeException $e ) 
					{
						unset( $popups[$k] );
					}
				}
				elseif ( $v[0] == "c" ) // chat popups
				{
					try
					{
						$con = \IPS\chatbox\Member\Conversation::load( $v[1] );
						
						if ( ! $con->_canChat() )
						{
							unset( $popups[$k] );
						}
						$html .= (string) $con->show();
					}
					catch( \OutOfRangeException $e ) 
					{
						unset( $popups[$k] );
					}					
				}
			}
			
			if ( \count( $popups ) > 0 )
			{
				\IPS\Request::i()->setCookie( $cookieName, implode("|", $popups), NULL, FALSE );
			}
			else
			{
				\IPS\Request::i()->setCookie( $cookieName, NULL );
			}
			
			return $html;
		}
	}
	 	
	/*-------------------------------------------------------------------------*/
	// Find existing conversation
	/*-------------------------------------------------------------------------*/	 
	public function getExistingConvo( $receiver )
	{
		$where = array( 'm.map_user_id=? OR m.map_user_id=?', \IPS\Member::loggedIn()->member_id, $receiver->member_id );
		
		try
		{
			$conID = (int) \IPS\Db::i()->select( 'm.map_con_id', array( 'chatbox_conversations_user_map', 'm' ), $where, NULL, NULL, 'm.map_con_id', 'COUNT(DISTINCT m.map_user_id)=2' )->first();
			$convo = \IPS\chatbox\Member\Conversation::load( $conID );
		}
		catch ( \UnderflowException $e )
		{
			$convo = \IPS\chatbox\Member\Conversation::createItem( \IPS\Member::loggedIn(), NULL, \IPS\DateTime::create(), NULL );
			$convo->title = \IPS\Member::loggedIn()->name . " - " . $receiver->name;
			$convo->save();
			
			\IPS\Db::i()->insert( 'chatbox_conversations_user_map', array(
				'map_user_id'		=> \IPS\Member::loggedIn()->member_id,
				'map_con_id'		=> $convo->id
			) );
			
			\IPS\Db::i()->insert( 'chatbox_conversations_user_map', array(
				'map_user_id'		=> $receiver->member_id,
				'map_con_id'		=> $convo->id
			) );				
		}

		return $convo;
	}

	/*-------------------------------------------------------------------------*/
	// Update last post ID
	/*-------------------------------------------------------------------------*/	 
	public function updateLastID()
	{
		$this->lastID = $this->comments(1, 0, 'id', 'desc')->id;
		$this->save();
	}
	
	/*-------------------------------------------------------------------------*/
	// Mark message as edited or deleted
	/*-------------------------------------------------------------------------*/
	public function markMsg( $id, $content=NULL )
	{
		$cache = array();
		
		if ( isset( \IPS\Data\Store::i()->chatbox_con_changes ) )
		{
			$cache = \IPS\Data\Store::i()->chatbox_con_changes;
		}
		
		foreach ( $this->maps() as $map )
		{
			if ( $map['map_user_id'] != \IPS\Member::loggedIn()->member_id )
			{
				$cache[ $map['map_user_id'] ][ $id ] = $content;
			}
		}

		\IPS\Data\Store::i()->chatbox_con_changes = $cache;
	}
	

	/*-------------------------------------------------------------------------*/
	// Cache latest ID
	/*-------------------------------------------------------------------------*/	
	public function cacheLatestID( $conID, $id=0 )
	{
		$cache = array();
		
		if ( isset( \IPS\Data\Store::i()->chatbox_con_msg ) )
		{
			$cache = \IPS\Data\Store::i()->chatbox_con_msg;
		}
		
		if ( !$cache[ $conID ] )
		{
			try
			{
				$row = \IPS\Db::i()->select( '*', 'chatbox_conversations_posts', array( 'con=?', $conID ), 'id DESC' )->first();
				$cache[ $conID ] = $row['id'];
			}
			catch ( \UnderflowException $e ) 
			{
				$cache[ $conID ] = 0;
			}
			\IPS\Data\Store::i()->chatbox_con_msg = $cache;
		}

		if ( $id > 0 && $cache[ $conID ] < $id )
		{
			$cache[ $conID ] = $id;
			\IPS\Data\Store::i()->chatbox_con_msg = $cache;
		}
	
		return $cache[ $conID ];
	}	
	
	
	
	
	
	
	
	
	
	
	
}