<?php

namespace IPS\chatbox\Rooms;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Package Item Model
 */
class _Chat extends \IPS\Content\Item
{
	use \IPS\Content\Reportable;
	
	/**
	 * @brief	Application
	 */
	public static $application = 'chatbox';
	
	/**
	 * @brief	Module
	 */
	public static $module = 'chatbox';
	
	/**
	 * @brief	Database Table
	 */
	public static $databaseTable = 'chatbox_messages';
	
	/**
	 * @brief	Database Prefix
	 */
	public static $databasePrefix = '';
	
	/**
	 * @brief	Node Class
	 */
	public static $containerNodeClass = 'IPS\chatbox\Rooms\Room';
	
	/**
	 * @brief	Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';
	
	/**
	 * @brief	Language prefix for forms
	 */
	public static $formLangPrefix = 'chatbox_';
	
	/**
	 * @brief	Title
	 */
	public static $title = 'chatbox_msg';
	
	/**
	 * @brief	Icon
	 */
	public static $icon = 'comment';
	
	/**
	 * @brief	[Content\Item]	Include the ability to search this content item in global site searches
	 */
	public static $includeInSiteSearch = FALSE;
	
	/**
	 * @brief	Database Column Map
	 */
	public static $databaseColumnMap = array(
		'container'				=> 'room',
		'author'				=> 'member_id',
		'date'					=> 'time',
		'title'					=> 'title',
		'content'				=> 'content',
	);	
	
	/**
	 * @brief	Cached URLs
	 */
	protected $_url	= array();
	
	/**
	 * @brief	URL Base
	 */
	public static $urlBase = 'app=chatbox&module=chatbox&controller=msg&id=';
	
	/**
	 * @brief	URL Template
	 */
	public static $urlTemplate = '';
	
	/**
	 * @brief	SEO Title Column
	 */
	public static $seoTitleColumn = '';
	
	/**
	 * Can move?
	 *
	 * @param	\IPS\Member|NULL	$member	The member to check for (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canMove( $member=NULL )
	{
		return FALSE;
	}
	
	/**
	 * Get elements for add/edit form
	 *
	 * @param	\IPS\Content\Item|NULL	$item				The current item if editing or NULL if creating
	 * @param	\IPS\Node\Model|NULL	$container			Container (e.g. forum), if appropriate
	 * @param	bool 					$fromCreateMenu		false to deactivate the minimize feature
	 * @return	array
	 */
	public static function formElements( $item=NULL, \IPS\Node\Model $container=NULL , $fromCreateMenu=FALSE)
	{
		$opts = $container->options();
		
		$formElements = parent::formElements( $item, $container );

		unset( $formElements['title'] );
		unset( $formElements['captcha'] );

		if ( \IPS\Request::i()->do != 'editEditor' )
		{
			$member = \IPS\Member::loggedIn();

			$minimize = ( $member->member_id != \IPS\Member::loggedIn()->member_id ) ?
				\IPS\Member::loggedIn()->language()->addToStack( static::$formLangPrefix . '_update_placeholder_other', FALSE, array( 'sprintf' => array( $member->name ) ) ) :
				static::$formLangPrefix . '_update_placeholder';
		}
		
		$editorName = 'chatMSG_' . $container->id;
		
		if( $item !== NULL )
		{
			$editorName .= '_' . $item->id;
		}
		
		$formElements['chatbox_content'] = new \IPS\Helpers\Form\Editor( $editorName, ( $item ) ? $item->content : NULL, TRUE, array(
			'app'			=> 'chatbox',
			'key'			=> 'Chat',
			'autoSaveKey' 	=> 'room_msg' . ( ( $item !== NULL ) ? '_' . $item->id : '' ),
			'minimize'		=> $minimize,
			'attachIds'		=> ( $item === NULL ) ? NULL : array( $item->id, NULL, $item->member_id ),
			'autoGrow'		=> TRUE,
			'maxLength'		=> $opts['length'] > 0 ? $opts['length'] : NULL,
		) );
		
		
		return $formElements;
	}
	
	/**
	 * Create from form
	 *
	 * @param	array					$values				Values from form
	 * @param	\IPS\Node\Model|NULL	$container			Container (e.g. forum), if appropriate
	 * @param	bool					$sendNotification	TRUE to automatically send new content notifications (useful for items that may be uploaded in bulk)
	 * @return	\IPS\Content\Item
	 */
	public static function createFromForm( $values, \IPS\Node\Model $container = NULL, $sendNotification = FALSE )
	{
		/* Create */
		$chat = parent::createFromForm( $values, $container, $sendNotification );
		
		/* Attachment */
		\IPS\File::claimAttachments( 'room_msg', $chat->id, NULL, $chat->member_id );
		
		/* Return */
		return $chat;
	}
	
	/**
	 * Process create/edit form
	 *
	 * @param	array				$values	Values from form
	 * @return	void
	 */
	public function processForm( $values )
	{
		parent::processForm( $values );
		
		if ( $this->_new )
		{
			$this->room = $values['chatbox_room'];
			$this->member_id = \IPS\Member::loggedIn()->member_id;
			$this->guestName = \IPS\Request::i()->cookie['chatbox_guestName'];
			$this->time = time();
		}
		
		if ( \IPS\Request::i()->_report > 0 )
		{
			$content = $values['chatbox_content_' . $this->id ];
		}
		else 
		{
			$content = $values['chatMSG_' . $this->container()->id];
		}
		
		$this->content = $content;
		$this->title = \IPS\Application::load('chatbox')->autoTitle( $content );
		$this->title_furl = \IPS\Http\Url\Friendly::seoTitle( $this->title );	
		
		$this->save();
	}	
	

	/**
	 * Should posting this increment the poster's post count?
	 *
	 * @param	\IPS\Node\Model|NULL	$container	Container
	 * @return	void
	 */
	public static function incrementPostCount( \IPS\Node\Model $container = NULL )
	{
		return FALSE;
	}
	
	/**
	 * Do not add sitemap
	 *
	 * @return	array
	 */
	public static function sitemapWhere()
	{
		return array( array( 'chatbox_messages.id=?', 0 ) );
	}

}