<?php
/**
 * @brief		chatboxWidget Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	chatbox
 * @since		03 Dec 2018
 */

namespace IPS\chatbox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * chatboxWidget Widget
 */
class _chatboxWidget extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'chatboxWidget';
	
	/**
	 * @brief	App
	 */
	public $app = 'chatbox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		$this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), $this->key ) );
		
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
		{
	 		$form = new \IPS\Helpers\Form;
 		}

		$options = array();
		
		foreach( \IPS\chatbox\Rooms\Room::roots() as $room )
		{
			$options[ $room->id ] = $room->_title;
		}

		$form->add( new \IPS\Helpers\Form\Select( 'room_to_show', isset( $this->configuration['room_to_show'] ) ? \intval( $this->configuration['room_to_show'] ) : 0, TRUE, array( 'options' => $options, 'parse' => 'normal' ) ) );	
				
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'devices_to_mini', isset( $this->configuration['devices_to_mini'] ) ? $this->configuration['devices_to_mini'] : array( 'Phone', 'Tablet', 'Desktop' ), FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'devices_to_mini' ) );

		$form->add( new \IPS\Helpers\Form\Radio( 'chatbox_where_to_show', $this->configuration['chatbox_where_to_show'], TRUE, array(
			'options' => array( 'all' => 'chatbox_where_to_show_all', 'pages' => 'chatbox_where_to_show_pages' ),
			'toggles' => array(
				'pages'	=> array( 'chatbox_pages' )
			)
		) ) );		
		
		$form->add( new \IPS\Helpers\Form\TextArea( 'chatbox_pages', isset( $this->configuration['chatbox_pages'] ) ? $this->configuration['chatbox_pages'] : $_SERVER["HTTP_REFERER"], FALSE, array('rows' => 3), null, null, null, 'chatbox_pages' ) );		
		
 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{			
 		return $values;
 	}

	/**
	 * Convert the widget to HTML
	 *
	 * @return	string
	 */
	public function __toString()
	{
		if ( $this->configuration['chatbox_where_to_show'] == 'pages' && $this->configuration['chatbox_pages'])
		{
			$currentpageURL = $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
			$pages = explode( "\n", str_replace( "\r", '', $this->configuration['chatbox_pages'] ) );
			foreach( $pages as $page )
			{
				$page = explode("://", $page);
				$pagelist[] = $page[1];
			}
			if ( ! \in_array( $currentpageURL, $pagelist ) )
			{
				return '';
			}
		}
		
		return parent::__toString();
	}
	
	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		try
		{
			$room = \IPS\chatbox\Rooms\Room::load( $this->configuration['room_to_show'] );
			
			if ( $room->_canView() )
			{
				return $this->output( $room, $this->configuration );
			}
		}
		catch( \OutOfRangeException $e ) {}
		
		return '';
	}
	
	/**
	 * Get HTML using the template (language strings not parsed)
	 *
	 * @return	string
	 */	
	public function output()
	{
		$args = \func_get_args();
		
		$args[] = $this->orientation;

		$output = \call_user_func_array( $this->template, $args );

		return $output;
	}	
}